package bg.bas.dcl.anaphora;

import java.io.File;
import java.io.FileOutputStream;
import java.io.IOException;
import java.io.OutputStreamWriter;
import java.io.Writer;
import java.text.DecimalFormat;
import java.util.ArrayList;
import java.util.Collections;
import java.util.HashMap;

import bg.bas.dcl.entities.AgreementObject;
import bg.bas.dcl.entities.Clause;
import bg.bas.dcl.entities.NounPhraseWords;
import bg.bas.dcl.entities.Sentence;
import bg.bas.dcl.entities.TextPosition;
import bg.bas.dcl.entities.Word;
import bg.bas.dcl.general.FileHandler;
import bg.bas.dcl.multilingual.Chunker.VerbFormRecogniser.ClauseVerb;
import bg.bas.dcl.multilingual.Chunker.VerbFormRecogniser.VerbFormFormalism;
import bg.bas.dcl.multilingual.ClauseSplitting.RuleBasedClauseSplitter;
import bg.bas.dcl.semantic.lexical_db.BulNet;
import bg.bas.dcl.semantic.processing.NounPhrase;
import bg.bas.dcl.xmlcorpusprocessor.XMLCorpusProcessor;

public class AnaphoraResolutor {
	
	final int CONTEXTSIZE = 1;
	final int NOUNSBACK = 6;
	
	XMLCorpusProcessor cpr = new XMLCorpusProcessor();
 BulNet bulnet ;
	
	RuleBasedClauseSplitter rbcs ;
	
	public AnaphoraResolutor(){
		
	}
	
	public static void main(String[] args){
		
		String dir = "corpus-tagged/";
		String outdir = "corpus-anaphora-1/";
		String bulnetf = "wnbg_7.06.2016.xml";

		
		String resources = "resources/";

		
		AnaphoraResolutor ar = new AnaphoraResolutor();
		
 	ar.bulnet = new BulNet(bulnetf);
		
		String modelsFile = resources + "/verb-form-models.txt";
		String conjFile = resources + "/conj.txt";
		String verbsFile = resources + "/special-verbs.txt";
		String punctExcFile = resources + "/strong-punct-exceptions.txt";

		VerbFormFormalism vff = new VerbFormFormalism();		
		vff.readModelsFromFile(modelsFile);

		ar.rbcs = new RuleBasedClauseSplitter(
				vff, conjFile, verbsFile, punctExcFile);
		
		ar.process(dir,outdir);
		
		
	}
	
	int ctrphi = 0;
	int ctrph = 0;
	int ctrse = 0;
	int ctrfpro = 0;
	int ctrpro = 0;
	int ctrsepp = 0;
	int ctrpp = 0;
	
	
	public void process(String dir, String outdir){
		
		try{
			
			FileHandler fh = new FileHandler();
			
			for (File f : fh.getFileListing(new File(dir))){
				
				if (!f.isFile()) continue;
				
				ArrayList<Sentence> sents = cpr.getSentencesFromFile(f);
				
				ArrayList<Sentence> resultsents = 
						processText(sents);
				
				String outf = f.getAbsolutePath()
						.replace(dir, outdir)
						+".csv";
				
				FileHandler.makeParent(outf, "");
				
				writeSentencesToFile(resultsents, outf);
				
			}
			
			
			System.out.println("PHi: " + ctrphi);
			System.out.println("PH: " + ctrph);
			System.out.println("Pse: " +  ctrse);
			System.out.println("PRO: " + ctrfpro);
			System.out.println("Otnos: " + ctrpro);
			System.out.println("PsePP: " + ctrsepp);
			System.out.println("PPP: " + ctrpp);
			

			
		} catch(Exception e){
			e.printStackTrace();
		}
		
	}
	
	
	public void writeSentencesToFile(
			ArrayList<Sentence> sents, 
			String outf){
		
		try{
			
			Writer wr = new OutputStreamWriter(
					new FileOutputStream(outf),"UTF-8");
			
			for (Sentence sent : sents){
				
				for (Word w : sent){
					String line = 
							w.clauseID + "\t"
							+ w.wordform + "\t"
									+ w.lemma + "\t"
									+ w.pos + "\t"
									+ w.wn + "\t"
									+ w.referentID + "\t"
									+ w.anaphoraID 
									+ "\n";
					line = line.replaceAll("\"", "&QUOT;");
					wr.write(
							line);
					wr.flush();
				}
			}
			
			wr.flush();
			wr.close();
			
		}catch(Exception e){
			e.printStackTrace();
		}
		
	}
	
	public ArrayList<Sentence> processText( ArrayList<Sentence> sents){
		
		ArrayList<TextPosition> NPs = new ArrayList<TextPosition>();
				
		for (int i = 0; i<sents.size(); i++) {
			
			Sentence sent = sents.get(i);
			
			ArrayList<ClauseVerb> verbs = 
					rbcs.getClauseVerbs(sent);
			sent = rbcs.performClauseSplitting(sent);
			
			int j = 0;
			
			HashMap<Integer,String> proind = new HashMap<Integer,String>();
m:			while(j<sent.size()) {
				
				Word word = sent.get(j);
				TextPosition tp = new TextPosition(i,j);
				
				for (ClauseVerb verb : verbs) {
										
					if (verb.get(0).sentpos == j){
						
						System.out.println(sent.getPlainText());
						System.out.println(sent.get(verb.get(0).sentpos).wordform);
						
						if(NPs.size()==0 
								|| i !=
										NPs.get(NPs.size()-1).parpos
								|| !sent.get(verb.get(0).sentpos).clauseID
							.equals(
							sent.get(NPs.get(NPs.size()-1).sentpos).clauseID)){
							
							boolean has = false;
							for (int t = verb.get(0).sentpos; t>0
									&& sent.get(t).clauseID.equals(sent.get(verb.get(0).sentpos).clauseID); t--){
								if (sent.get(t).pos.startsWith("N")) has = true;
								if (sent.get(t).pos.startsWith("PHi")) has = true;
								if (sent.get(t).pos.startsWith("PRO")
										&& t>0
										&& !sent.get(t-1).pos.startsWith("R")) has = true;
							}
							for (int t = verb.get(0).sentpos; t<sent.size()
									&& sent.get(t).clauseID.equals(sent.get(verb.get(0).sentpos).clauseID); t++){
								if (sent.get(t).pos.startsWith("N")
										&& sent.get(t).pos.contains("l")) has = true;
								if (sent.get(t).pos.startsWith("A")
										&& sent.get(t).pos.contains("l")) has = true;
								if (sent.get(t).pos.startsWith("N")
										&& verb.tense.contains("Pas")) has = true;
								if (sent.get(t).pos.startsWith("PHi")) has = true;
								if (sent.get(t).pos.startsWith("PRO")
										&& t>0
										&& !sent.get(t-1).pos.startsWith("R")) has = true;
							}
							String per = "???";
							for (TextPosition vtp : verb){
								String vpos = sent.get(vtp.sentpos).pos;
								if (vpos.contains("1") && vpos.contains("s")){
									per = "1s";
									break;
								}
								if (vpos.contains("1") && vpos.contains("p")){
									per = "1p";
									break;
								}
								if (vpos.contains("2") && vpos.contains("s")){
									per = "2s";
									break;
								}
								if (vpos.contains("2") && vpos.contains("p")){
									per = "2p";
									break;
								}
								if (vpos.contains("s")){
									per = "3s???";
									break;
								}
								if (vpos.contains("p")){
									per = "3p???";
									break;
								}
								
								
							}
							
							if (per.contains("3")){
								ArrayList<AnaphoraReferentCandidate>
								list = findPROMatchIndex(
										"FPRO"+per, NPs, sents, i, j);
								
								int index = findBestCandidate(list);
								
								String scores = "";
								
								
								for (AnaphoraReferentCandidate cc : list) {
									if (cc.score<0.1) continue;
									scores += cc.NPid+":"+
											new DecimalFormat("#.00").format(cc.score)
								+":"+cc.clausedistance+";";
								}
								
								per = index + "\t"+scores;
							}
							
							if (!has) {
								proind.put(j,per);
								ctrfpro ++;
							}
							j++;
							
							continue m;
						}
				
					}
				}
				
				if (!word.pos.startsWith("N")
						&& bulnet.containsKey(word.wn)
						&& bulnet.get(word.wn).pos.equalsIgnoreCase("n")){
					word.pos = "N" + word.pos;
				}
				
				
				if (word.pos.startsWith("N")){
					
					if (j==0 || !sents.get(i).get(j-1).wn.equals(word.wn)) {
						NPs.add(tp);
					} 
					
					String tindex = (NPs.size()-1)+"";
					while (tindex.length() < 4) tindex = "0"+tindex;
					
					word.referentID = tindex;
					j++;
					continue;
				}
				
				if (word.lemma.equals("се")
						|| word.lemma.equals("си") ) ctrse ++;
				if (word.lemma.equals("свой") ) ctrsepp ++;

				
				if (isAnaphora(word.pos)){
					
					ArrayList<AnaphoraReferentCandidate>
					list = findAdvancedAnaphoraMatchIndex(
							word.pos, NPs, sents, i, j);
					
					int index = findBestCandidate(list);
					
					String scores = "";
					
					for (AnaphoraReferentCandidate cc : list) {
						if (cc.score<0.1) continue;
						scores += cc.NPid+":"+
								new DecimalFormat("#.00").format(cc.score)
					+":"+cc.clausedistance+";";
					}
					 
						
					if (index > -1) {
						TextPosition refposition = NPs.get(index);

						String tindex = index+"";
						while (tindex.length() < 4) tindex = "0"+tindex;
						word.anaphoraID = tindex+"\t"+scores;
						
						Word ref = sents.get(refposition.parpos)
								.get(refposition.sentpos);
						ref.referentID = tindex;
					}
	
					
					if (word.pos.startsWith("PHi3")) ctrphi ++;
					if (word.pos.startsWith("PRO")) ctrpro ++;
					if (word.pos.startsWith("PHya3")) ctrph ++;
					if (word.pos.startsWith("PHza3")) ctrph ++;
					if (word.pos.startsWith("PPy")
							&& word.pos.contains("q")) ctrpp ++;
					if (word.pos.startsWith("PPz")
							&& word.pos.contains("q")) ctrpp ++;

					if (word.lemma.equals("се")
							|| word.lemma.equals("си") ) ctrse ++;
					if (word.lemma.equals("свой") ) ctrsepp ++;

				}

				j++;
				
			}
			
			ArrayList<Integer> proindsorted = new ArrayList<Integer>();
			proindsorted.addAll(proind.keySet());
			Collections.sort(proindsorted);
			
			for (int k = 0; k<proindsorted.size(); k++){
				Word pro = new Word();
				pro.wordform = "FPRO";
				pro.lemma = "FPRO";
				pro.pos = "FPRO";
				pro.clauseID = sent.get(k+proindsorted.get(k)).clauseID;
				
				pro.anaphoraID = proind.get(proindsorted.get(k));

				sent.add(k+proindsorted.get(k), pro);
			}
			
			
		}
		
		return sents;
		
	}
	
	public int findBestCandidate(ArrayList<AnaphoraReferentCandidate> list){
		
		int ind = -1;
		double max = 0.0;
		
		for (AnaphoraReferentCandidate cand : list){
			
			if (cand.score > max) {
				ind = cand.NPid;
				max = cand.score;
			}
		
			// get first candidate
//			ind = cand.NPid;
//			break;
			
		}
		
		return ind;
		
	}
	
	public int findAnaphoraMatchIndex(String pos, 
			ArrayList<TextPosition> NPs,
			ArrayList<Sentence> sents, int i, int j){
	
		int init = NPs.size()-1;
		
		if (pos.startsWith("PPz")){
		
//			System.out.println(i+" "+j + " " 
//					+ NPs.get(init).parpos + " "+ NPs.get(init).sentpos);

				if (init>-1 
						&& i == NPs.get(init).parpos
				&& j == NPs.get(init).sentpos+1){
			init = init - 1;
				}
		}
		
		if (pos.startsWith("PHy")
				|| pos.startsWith("PHz")){
			
//			System.out.println(i+" "+j + " " 
//					+ NPs.get(init).parpos + " "+ NPs.get(init).sentpos);

				if ( init>-1 
						&& i == NPs.get(init).parpos
				&& j == NPs.get(init).sentpos+2
				&& sents.get(i).get(j-1).pos.equals("R")){
			init = init - 1;
				}
		}

		
		if (pos.startsWith("PPy")
				|| pos.startsWith("PPz")){
			pos = pos
					.replace("s", "")
					.replace("p", "")
					.replace("m", "")
					.replace("n", "")
					.replace("f", "")
					.replace("s", "")
					
					.replace("g", "s")
					.replace("u", "p")
					.replace("b", "f")
					.replace("c", "n")
					.replace("e", "m")
					;
		}
		
		
		for (int k = init; k>0; k--){
			
			TextPosition position = NPs.get(k);
			
			Word np = sents.get(position.parpos)
					.get(position.sentpos);
			
			if (checkAgreement(pos,np.pos)){
				return k;
			}
			
		}
		
		return -1;
	}
	
	
	
	public ArrayList<AnaphoraReferentCandidate> 
			findAdvancedAnaphoraMatchIndex(String pos, 
					ArrayList<TextPosition> NPs,
					ArrayList<Sentence> sents, int i, int j){
	
		ArrayList<AnaphoraReferentCandidate> list = new ArrayList<AnaphoraReferentCandidate>();
		
		int inbegin = NPs.size()-1;
		int inend = 0;
		
		if (pos.startsWith("PHi")
				|| pos.startsWith("PRO")){
			
//			System.out.println(i+" "+j + " " 
//					+ NPs.get(init).parpos + " "+ NPs.get(init).sentpos);

			while (inbegin >-1 
					&& NPs.get(inbegin).sentpos == j
					&& sents.get(NPs.get(inbegin).parpos)
						.get(NPs.get(inbegin).sentpos).clauseID.equals(
								sents.get(i).get(j).clauseID)){
				inbegin --;
			}
			

		}

		
		if (pos.startsWith("PPz")){
		
//			System.out.println(i+" "+j + " " 
//					+ NPs.get(init).parpos + " "+ NPs.get(init).sentpos);

				if (inbegin>-1 
						&& i == NPs.get(inbegin).parpos
				&& j == NPs.get(inbegin).sentpos+1){
					inbegin = inbegin - 1;
				}
		}
		
		if (pos.startsWith("PHy")
				|| pos.startsWith("PHz")){
			
//			System.out.println(i+" "+j + " " 
//					+ NPs.get(init).parpos + " "+ NPs.get(init).sentpos);

				if ( inbegin>-1 
						&& i == NPs.get(inbegin).parpos
				&& j == NPs.get(inbegin).sentpos+2
				&& sents.get(i).get(j-1).pos.equals("R")){
					inbegin = inbegin - 1;
				}
		}
		
		inend = inbegin;
		while (inend >-1 
				&& 
				NPs.get(inend).parpos - i<3){
			inend --;
		}


		
		if (pos.startsWith("PPy")
				|| pos.startsWith("PPz")){
			pos = pos
					.replace("s", "")
					.replace("p", "")
					.replace("m", "")
					.replace("n", "")
					.replace("f", "")
					.replace("s", "")
					
					.replace("g", "s")
					.replace("u", "p")
					.replace("b", "f")
					.replace("c", "n")
					.replace("e", "m")
					;
		}
		
		
		for (int k = inbegin; k>0; k--){
			
			TextPosition position = NPs.get(k);
			
			Word np = sents.get(position.parpos)
					.get(position.sentpos);
			
			if (checkAgreement(pos,np.pos)){
				AnaphoraReferentCandidate cand = new AnaphoraReferentCandidate();
				cand.NPid = k;
				cand.score = 10.0;
				
				if (hasRepetition(NPs, sents, inbegin, inend, np.lemma)){
					cand.score *= 1.1;
				}
				if (hasSameCollocations(position, sents, i, j)){
					cand.score *= 1.1;
				}
				if (hasSimilarSyntax(position, sents, i, j)){
					cand.score *= 1.05;
				}
				if (hasAdjective(position, sents)){
					cand.score *= 1.05;
				}
				if (isDefinite(position, sents)){
					cand.score *= 1.1;
				}
				if (isNE(position, sents)){
					cand.score *= 1.1;
				}
				if (isPP(position, sents)){
					cand.score *= 0.9;
				}
				
				int clctr = getClauseDistance(position,sents,i,j);
				cand.score *= Math.max(0.0,1.0-Math.abs(clctr-1)*0.15);
				cand.clausedistance = clctr;

				list.add(cand);
			}
			
		}
		
		return list;
	}
	
	
	public ArrayList<AnaphoraReferentCandidate> 
		findPROMatchIndex(String pos, 
			ArrayList<TextPosition> NPs,
			ArrayList<Sentence> sents, int i, int j){

		ArrayList<AnaphoraReferentCandidate> list = new ArrayList<AnaphoraReferentCandidate>();
		
		int inbegin = NPs.size()-1;
		int inend = 0;
		
			while (inbegin >-1 
					&& NPs.get(inbegin).sentpos == j
					&& sents.get(NPs.get(inbegin).parpos)
						.get(NPs.get(inbegin).sentpos).clauseID.equals(
								sents.get(i).get(j).clauseID)){
				inbegin --;
			}
		
		
		inend = inbegin;
		while (inend >-1 
				&& 
				NPs.get(inend).parpos - i<3){
			inend --;
		}
		
		
	
		
		for (int k = inbegin; k>0; k--){
			
			TextPosition position = NPs.get(k);
			
			Word np = sents.get(position.parpos)
					.get(position.sentpos);
			
			if (checkAgreement(pos,np.pos)){
				AnaphoraReferentCandidate cand = new AnaphoraReferentCandidate();
				cand.NPid = k;
				cand.score = 10.0;
				
				if (hasRepetition(NPs, sents, inbegin, inend, np.lemma)){
					cand.score *= 1.3;
				}
				if (hasSameCollocations(position, sents, i, j)){
					cand.score *= 1.2;
				}
				if (hasSimilarSyntax(position, sents, i, j)){
					cand.score *= 3;
				}
				if (hasAdjective(position, sents)){
					cand.score *= 1.2;
				}
				if (isDefinite(position, sents)){
					cand.score *= 1.2;
				}
				if (isNE(position, sents)){
					cand.score *= 1.2;
				}
				if (isPP(position, sents)){
					cand.score *= 0.8;
				}
				
				int clctr = getClauseDistance(position,sents,i,j);
				cand.score *= Math.max(0.0,1.0-Math.abs(clctr-1)*0.1);
				cand.clausedistance = clctr;
		
				list.add(cand);
			}
			
		}
		
		return list;
		}
		

	
	
	public int getClauseDistance(TextPosition tp,
			ArrayList<Sentence> sents, int i, int j){
		int ctr = 0;
	
		for (int k = i; k>tp.parpos-1; k--){
			for (int l = k==i?j:sents.get(k).size()-1; l>(k==tp.parpos?tp.sentpos:-1); l--){
				
				if (l==sents.get(k).size()-1) ctr++;
				else if (!sents.get(k).get(l+1).clauseID
						.equals(sents.get(k).get(l).clauseID)) ctr++;
				
			}
			
		}
		
		return ctr;
	}

	public boolean isNE(TextPosition tp,
			ArrayList<Sentence> sents){
		
		if (sents.get(tp.parpos).get(tp.sentpos).pos.startsWith("NH")){
			return true;
		}
		if (tp.sentpos > 0
				&& !sents.get(tp.parpos).get(tp.sentpos).wordform.toLowerCase()
					.equals(sents.get(tp.parpos).get(tp.sentpos).wordform)){
			return true;
		}


		return false;
	}
	
	public boolean isPP(TextPosition tp,
			ArrayList<Sentence> sents){
		
		if (tp.sentpos > 0
				&& sents.get(tp.parpos).get(tp.sentpos-1).pos.equals("R")){
			return true;
		}
		if (tp.sentpos > 1
				&& sents.get(tp.parpos).get(tp.sentpos-1).pos.startsWith("A")
				&& sents.get(tp.parpos).get(tp.sentpos-2).pos.startsWith("R")){
			return true;
		}

		if (tp.sentpos > 2
				&& sents.get(tp.parpos).get(tp.sentpos-1).pos.startsWith("A")
				&& sents.get(tp.parpos).get(tp.sentpos-2).pos.startsWith("A")
				&& sents.get(tp.parpos).get(tp.sentpos-3).pos.startsWith("R")){
			return true;
		}


		return false;
	}


	public boolean isDefinite(TextPosition tp,
			ArrayList<Sentence> sents){
		String pos = sents.get(tp.parpos).get(tp.sentpos).pos;
		if (pos.contains("d") || pos.contains("l") || pos.contains("h")){
			return true;
		}

		if (tp.sentpos > 0
				&& sents.get(tp.parpos).get(tp.sentpos-1).pos.startsWith("A")){
			String pos1 = sents.get(tp.parpos).get(tp.sentpos-1).pos;
			if (pos1.contains("d") || pos1.contains("l") || pos1.contains("h")){
				return true;
			}
		}
		if (tp.sentpos > 1
				&& sents.get(tp.parpos).get(tp.sentpos-1).pos.startsWith("A")
				&& sents.get(tp.parpos).get(tp.sentpos-2).pos.startsWith("A")){
			String pos2 = sents.get(tp.parpos).get(tp.sentpos-2).pos;
			if (pos2.contains("d") || pos2.contains("l") || pos2.contains("h")){
				return true;
			}
		}

		return false;
	}
	
	public boolean hasAdjective(TextPosition tp,
			ArrayList<Sentence> sents){
		
		if (tp.sentpos > 0
				&& sents.get(tp.parpos).get(tp.sentpos-1).pos.startsWith("A")){
			return true;
		}

		return false;
	}

	
	public boolean hasSimilarSyntax(TextPosition tp,
			ArrayList<Sentence> sents, int i, int j){
		
		if (j>0 && tp.sentpos > 0
				&& sents.get(i).get(j-1).pos.startsWith("V")
				&& sents.get(tp.parpos).get(tp.sentpos-1).pos.startsWith("V")){
			return true;
		}
		if (j>0 && tp.sentpos > 0
				&& sents.get(i).get(j-1).pos.startsWith("R")
				&& sents.get(tp.parpos).get(tp.sentpos-1).pos.startsWith("R")){
			return true;
		}
		if (j<sents.get(i).size()-1 && tp.sentpos < sents.get(tp.parpos).size()-1
				&& sents.get(i).get(j+1).pos.startsWith("V")
				&& sents.get(tp.parpos).get(tp.sentpos+1).pos.startsWith("V")){
			return true;
		}

		return false;
	}

	
	public boolean hasSameCollocations(TextPosition tp,
			ArrayList<Sentence> sents, int i, int j){
		
		if (j>0 && tp.sentpos > 0
				&& sents.get(i).get(j-1).lemma
					.equals(sents.get(tp.parpos).get(tp.sentpos-1).lemma)){
			return true;
		}

		if (j<sents.get(i).size()-1 && tp.sentpos < sents.get(tp.parpos).size()-1
				&& sents.get(i).get(j+1).lemma
				.equals(sents.get(tp.parpos).get(tp.sentpos+1).lemma)){
			return true;
		}

		return false;
	}


	public boolean hasRepetition(ArrayList<TextPosition> NPs,
			ArrayList<Sentence> sents, int begin, int end, String lemma){
		
		int ctr = 0;
		for (int i = begin; i>=end&&i>-1; i--){
			TextPosition tp = NPs.get(i);
			if (sents.get(tp.parpos).get(tp.sentpos).lemma.equalsIgnoreCase(lemma)) ctr++;
		}
		
		if (ctr>1) return true;
		else return false;
	}

	
	public boolean isAnaphora(String pos){
		
		if (pos.startsWith("PHi3")) return true;
		if (pos.startsWith("PRO")) return true;
		if (pos.startsWith("FPRO")) return true;
		if (pos.startsWith("PHya3")) return true;
		if (pos.startsWith("PHza3")) return true;
		if (pos.startsWith("PPy")
				&& pos.contains("q")) return true;
		if (pos.startsWith("PPz")
				&& pos.contains("q")) return true;
		
		return false;
	}

	/**
	public ArrayList<NounPhrase> resolve(ArrayList<Sentence> sents, 
			ArrayList<NounPhrase> nouns) throws IOException{
		
		for (int i = 0; i<nouns.size(); i++) {
			
			if (!nouns.get(i).type.startsWith("ANN")) continue;
			
			for (int j = i-1; j>Math.max(0,i-NOUNSBACK-1); j--){
				
//				if (!nouns.get(j).type.contains("NP")) continue;
								
				boolean isMatch = checkAgreement(nouns.get(i), nouns.get(j));
				
				if (isMatch) {
	
					
					System.err.println("MATCH "+i+"\t"
							+ sents.get(nouns.get(i).get(0).parpos)
								.get(nouns.get(i).get(0).sentpos).wordform
								+"\t" + j 
								+"\t" + nouns.get(i).type+nouns.get(i).gram
								+"\t" + sents.get(nouns.get(j).get(0).parpos)
							.get(nouns.get(j).get(0).sentpos).wordform 
							+"\t" + nouns.get(j).type+nouns.get(j).gram);
				
					
					if (nouns.get(j).type.startsWith("ANN")) {
						nouns.get(i).referent = nouns.get(j).referent;
					} else {
						nouns.get(i).referent = nouns.get(j);
					}
					
					break;
				}
				
			}
			
		}
		return nouns;
		
	}
	*/
	public boolean checkAgreement(String pos1, String pos2){
			
		if (pos1.startsWith("N")
				&& pos1.contains("c")) pos1 = pos1.replace("c", "p");
		if (pos2.startsWith("N")
				&& pos2.contains("c")) pos2 = pos2.replace("c", "p");
		
		if (pos1.startsWith("PHy")
				&& (pos1.contains("m") || pos1.contains("n"))) pos1 = pos1+"nm";
		if (pos2.startsWith("PHy")
				&& (pos2.contains("m") || pos2.contains("n"))) pos2 = pos2+"nm";
		
		if (pos1.startsWith("PHz")
				&& (pos1.contains("m") || pos1.contains("n"))) pos1 = pos1+"nm";
		if (pos2.startsWith("PHz")
				&& (pos2.contains("m") || pos2.contains("n"))) pos2 = pos2+"nm";

		if (pos1.startsWith("PPy")
				&& (pos1.contains("m") || pos1.contains("n"))) pos1 = pos1+"nm";
		if (pos2.startsWith("PPy")
				&& (pos2.contains("m") || pos2.contains("n"))) pos2 = pos2+"nm";

		if (pos1.startsWith("PPz")
				&& (pos1.contains("m") || pos1.contains("n"))) pos1 = pos1+"nm";
		if (pos2.startsWith("PPz")
				&& (pos2.contains("m") || pos2.contains("n"))) pos2 = pos2+"nm";

			String[] gens = {"m","n","f"};
			String[] nums = {"s","p"};
			
			String gram1 = pos1;
			String gram2 = pos2;

			String n1 = "", n2 = "";
			String g1 = "", g2 = "";
	
			for (String n : nums) {
					if (gram1.contains(n)) n1 = n;
					if (gram2.contains(n)) n2 = n;
					
					if (!n1.equals("")
							&& !n2.equals("")) break;
			}
			
			for (String g : gens) {
				if (gram1.contains(g)) g1 = g;
				if (gram2.contains(g)) g2 = g;
				
				if (!g1.equals("")
						&& !g2.equals("")) break;
			}
			
			boolean result = true;
			
			if (n1.length()+n2.length()==1
					&& g1.length()+g2.length()==1) result=false;
			if (!n1.equals("") && !n2.equals("") && !n1.equals(n2)) result= false;
			if (!g1.equals("") && !g2.equals("") && !g1.equals(g2)) result=false;

			/**
			System.out.println(pos1+" "+pos2 + " "+result);
			try {
				System.in.read();
			} catch (IOException e) {
				// TODO Auto-generated catch block
				e.printStackTrace();
			}
			*/

			return result;


	}
	
	/**
		
f0:		for (int i = 0; i < clauseOrder.size(); i++){
			
System.err.println("CL:"+ clauseOrder.get(i));
			Clause cl = clauses.get(clauseOrder.get(i));

			int verbIndex = senAn.getVerbIndex(cl);
			
			AgreementObject anaphoraAgreement = new AgreementObject();
			
f1:			for (int j = 0; j < cl.size(); j++){	
				
				if (cl.get(j).pos.startsWith("PRO")
						|| cl.get(j).pos.startsWith("PP")
						|| cl.get(j).pos.startsWith("PH")
						) {
					
					System.err.println("PROCESSED "+cl.get(j).wordform 
							+ ","+cl.get(j).pos+","+cl.get(j).wordID);
					
					AgreementObject proAgreement = new AgreementObject();
					proAgreement.setGenderFromWord(cl.get(j));
					proAgreement.setNumberFromWord(cl.get(j));
					proAgreement.setPersonFromWord(cl.get(j));
				
					if (cl.get(j).pos.startsWith("PP")) 
						proAgreement.setGender(
							(cl.get(j).pos.endsWith("ge")? "mn" :
								cl.get(j).pos.endsWith("gb")? "f" : "")
							);
					if (cl.get(j).pos.startsWith("PP")) 
						proAgreement.setNumber(
							(cl.get(j).pos.endsWith("u")? "p" : "s")
							);					

					System.err.println("PROCESSED "+proAgreement.gender 
							+ ","+proAgreement.number);
										
					ArrayList<NounPhraseWords> list = senAn.getNounPhrases (cl);
					System.err.println("LIST SIZE "+list.size());
					
					for (NounPhraseWords f : list) System.err.println(f.head.wordform+","+f.head.wordID);
					
					for (int k = list.size()-1; k >= 0; k--){
						
						if (list.get(k).start > j) continue;
						
						AgreementObject npAgreement = new AgreementObject();
						
						if (cl.get(j).pos.startsWith("PRO")){
						}
						
//						if (cl.get(j).pos.startsWith("PP")){
							npAgreement.setGenderFromWord(list.get(k).head);
							npAgreement.setNumberFromWord(list.get(k).head);
							npAgreement.setPersonFromWord(list.get(k).head);
//						}
//						if (cl.get(j).pos.startsWith("PH")){
//							npAgreement.setGenderFromWord(list.get(k).head);
//							npAgreement.setNumberFromWord(list.get(k).head);
//							npAgreement.setPersonFromWord(list.get(k).head);
//						}

						if (cl.get(j).wordID - list.get(k).head.wordID==1) continue;
						if (j>2 && cl.get(j-1).pos.equals("R") 
								&& cl.get(j).wordID - list.get(k).head.wordID==2) continue;
						
						System.err.println("HEAD "+list.get(k).head.wordform 
								+ ","+list.get(k).head.pos+","+list.get(k).head.wordID);
						System.err.println("HEAD "+npAgreement.gender 
								+ ","+npAgreement.number+","+list.get(k).head.wordID);

						if (npAgreement.agreesWith(proAgreement)
								&& cl.get(j).wordID > list.get(k).head.wordID) {
							
							cl.get(j).reference = list.get(k).head;;

									System.err.println("MATCH " +cl.get(j).wordform+" "+
											list.get(k).head.wordform );
									continue f1;
						}

					}
					
					
					
					for (int l = 1; l < 5; l++){
						
						if (i<l) continue f1;
						
						Clause clprev = clauses.get(clauseOrder.get(i-l));
						System.out.println(clprev.toString());
						
						ArrayList<NounPhraseWords> listprev = senAn.getNounPhrases (clprev);
						System.out.println(listprev.size());
						
						for (int k = listprev.size()-1; k >= 0; k--){
							
							AgreementObject npAgreement = new AgreementObject();
//							if (cl.get(j).pos.startsWith("PRO")){
								npAgreement.setGenderFromWord(listprev.get(k).head);
								npAgreement.setNumberFromWord(listprev.get(k).head);
								npAgreement.setPersonFromWord(listprev.get(k).head);
//							}
							
//							if (cl.get(j).pos.startsWith("PP")){
//							}

//							if (cl.get(j).pos.startsWith("PH")){
//								npAgreement.setGenderFromWord(listprev.get(k).head);
//								npAgreement.setNumberFromWord(listprev.get(k).head);
//								npAgreement.setPersonFromWord(listprev.get(k).head);
//							}
								System.out.println(npAgreement.gender + "\t" + npAgreement.number 
										+ "\t" + npAgreement.person);
							if (npAgreement.agreesWith(proAgreement)
									&& cl.get(j).wordID > listprev.get(k).head.wordID) {
								cl.get(j).reference = listprev.get(k).head;

								System.err.println("MATCH " +cl.get(j).wordform+" "+
										listprev.get(k).head.lemma );
								continue f1;
							}
	
						}
					}
					
				}
				

			}
			
		}
		

		for (int i = 0; i < clauseOrder.size(); i++){
			
			Clause cl = clauses.get(clauseOrder.get(i));
			
			for (Word w : cl) {
				
				if (w.pos.startsWith("PH")
						|| w.pos.startsWith("PP")
						|| w.pos.startsWith("PRO")){
					out.write("<font color=\"red\"><b>"+w.wordform+"</b></font>");
					
				} else {
					out.write(w.wordform);
				}
				
				if (w.reference != null) {
					out.write("(" + w.reference.wordform + ")");
				}
//				out.write(","+w.pos+","+w.wordID);
				out.write(" ");
			}
			
			out.write("\n<br><br>\n");
			
		}		
		
		return clauses;
	*/


}
