package bg.bas.dcl.monolingual.bg;

import java.io.IOException;
import java.io.Writer;
import java.util.ArrayList;
import java.util.HashMap;

import bg.bas.dcl.entities.AgreementObject;
import bg.bas.dcl.entities.Clause;
import bg.bas.dcl.entities.NounPhraseWords;
import bg.bas.dcl.entities.Sentence;
import bg.bas.dcl.entities.Verb;
import bg.bas.dcl.entities.Word;

public class SentenceAnalyser {
	
	
	public NounPhraseWords getSubject (Clause cl, Clause clprev, ArrayList<NounPhraseWords> NPs){
		
		NounPhraseWords subject = new NounPhraseWords();
		int verbIndex =  getVerbIndex(cl);
		
		for (NounPhraseWords np : NPs){
			
			if (np.start > verbIndex) break;
			
			subject = np;
			
			// get phrases with PP: NP p NP
			break;
			
		}
		
		if (subject.size() < 1) {
			subject = getPronounSubject(cl);
		}
		
		if (subject.size() < 1) {
			subject = recoverProSubject(cl, clprev);
			cl.subject = subject;
		}

		return subject;
	}
	
	public NounPhraseWords getPronounSubject (Clause cl){
		
		NounPhraseWords subject = new NounPhraseWords();
		Verb verb = getVerbForm(cl);
		int verbIndex =  getVerbIndex(cl);
		
		for (int j = 0; j < verbIndex; j++){
			if (cl.get(j).pos.startsWith("PH")) {
				subject.add(cl.get(j));
				subject.start = j;
				subject.end = j;
				subject.head = cl.get(j);
				subject.headID = j;
			}
		}
		
		return subject;
	}
	
	public NounPhraseWords recoverProSubject (Clause cl, Clause clprev){
		
		NounPhraseWords subject = new NounPhraseWords();
		Verb verb = getVerbForm(cl);
		
		for (int j = 0; j < cl.size(); j++){

			if (verb.agreementObject.person.equals("1s")) {
				Word w = new Word();	
				w.wordform = "аз";
				w.lemma = "аз";
				w.pos = "PHi1s";
				w.clauseID = cl.get(0).clauseID;

			} 
			if (verb.agreementObject.person.equals("2s")) {
				Word w = new Word();	
				w.wordform = "ти";
				w.lemma = "ти";
				w.pos = "PHi2s";
				w.clauseID = cl.get(0).clauseID;

			} 
			if (verb.agreementObject.person.equals("1p")) {
				Word w = new Word();	
				w.wordform = "ние";
				w.lemma = "ние";
				w.pos = "PHi1p";
				w.clauseID = cl.get(0).clauseID;

			} 
			if (verb.agreementObject.person.equals("2p")) {
				Word w = new Word();	
				w.wordform = "вие";
				w.lemma = "вие";
				w.pos = "PHi2p";
				w.clauseID = cl.get(0).clauseID;

			} 

			
				if (verb.agreementObject.person.equals("3s")
						|| verb.agreementObject.person.equals("3p")) {
					
					if (clprev==null ||clprev.size() == 0) break; 
					
					subject = getSubject(clprev, null, getNounPhrases (clprev));
					AgreementObject subjectAgreement = new AgreementObject();
					subjectAgreement.setGenderFromWord(subject.head);
					subjectAgreement.setNumberFromWord(subject.head);
					if (subjectAgreement.person.equals("")) 
						subjectAgreement.person = 
								"3" + subjectAgreement.number.substring(1);
					
					if (subjectAgreement.agreesWith(verb.agreementObject)) {
						
						return subject;

					}

				} else {
					
//					System.err.println(subject.lemma);
					continue;
				}


				
				break;
			}
			

		return null;
	}
	
	public void recoverProSubjects (HashMap<String,Clause> list, 
			ArrayList<String> clauseOrder, Writer out) throws IOException{
		
		NounPhraseWords subject = new NounPhraseWords();

main:	for (int i = 0; i < clauseOrder.size(); i++) {
			
			Clause cl = list.get(clauseOrder.get(i));
		
			out.write("\n<br>***<br>\n");
			out.write(cl.toString());
			
			Verb verb = getVerbForm(cl);
			
			subject = getSubject(cl, null, getNounPhrases(cl));
			

			if (subject == null) subject = new NounPhraseWords();
			
			if (subject!=null && subject.size() > 0) {
				System.out.println("SUBJECT " + subject.head.wordform);
				out.write("\n<b>" 
						+subject.head.wordform + "</b>\n");

				continue;
			}
			
			out.write("\nVERB AGGREEMENT: " 
					+(verb.size()>0?verb.get(0).wordform:"")+" "
					+verb.agreementObject.person
					+ " " + verb.agreementObject.gender
					+" " + verb.agreementObject.number+ "\n");


						
			System.out.println("SUBJECT none");
			
			for (int j = 0; j < cl.size(); j++){
	
				if (verb.agreementObject.person.equals("1s")) {
					Word w = new Word();	
					w.wordform = "аз";
					w.lemma = "аз";
					w.pos = "PHi1s";
					w.clauseID = cl.get(0).clauseID;
					subject.add(w);
	
				} 
				if (verb.agreementObject.person.equals("2s")) {
					Word w = new Word();	
					w.wordform = "ти";
					w.lemma = "ти";
					w.pos = "PHi2s";
					w.clauseID = cl.get(0).clauseID;
					subject.add(w);
	
				} 
				if (verb.agreementObject.person.equals("1p")) {
					Word w = new Word();	
					w.wordform = "ние";
					w.lemma = "ние";
					w.pos = "PHi1p";
					w.clauseID = cl.get(0).clauseID;
					subject.add(w);
	
				} 
				if (verb.agreementObject.person.equals("2p")) {
					Word w = new Word();	
					w.wordform = "вие";
					w.lemma = "вие";
					w.pos = "PHi2p";
					w.clauseID = cl.get(0).clauseID;
					subject.add(w);
	
				} 
				
				if (subject != null && subject.size()>0) {
					out.write("<font color=\"red\">" 
							+subject.get(0).wordform + "</font>\n");

					out.flush();
					continue main;
				}
	
				
					if (verb.agreementObject.person.equals("3s")
							|| verb.agreementObject.person.equals("3p")) {
						
						
						for (int l = 1; l < 3; l++){
							if (i-l < 0) break;
							
							Clause clprev = list.get(clauseOrder.get(i-l));
							if (clprev==null || clprev.size() == 0) break; 
							
							subject = getSubject(clprev, null, getNounPhrases (clprev));
							
							if (subject == null || subject.size()==0) continue;
							out.write("<font color=\"red\">" 
									+"POSSIBLE SUBJECT: "
									+subject.get(0).wordform+ "</font>\n");
							
							AgreementObject subjectAgreement = new AgreementObject();
							subjectAgreement.setGenderFromWord(subject.head);
							subjectAgreement.setNumberFromWord(subject.head);
							if (subjectAgreement.person.equals("")) 
								subjectAgreement.person = 
										"3" + subjectAgreement.number.substring(0);
							
							if (subjectAgreement.agreesWith(verb.agreementObject)) {
								
								out.write("<font color=\"red\">" 
										+subject.head.wordform + "</font>\n");
								out.flush();
								
								continue main;
		
							}
						}
	
					} else {
						
	//					System.err.println(subject.lemma);
						continue;
					}
	
	
					
					break;
				}
				

		}
		
	}


	public int getVerbIndex (Clause cl){
		
		
		for (int j = 0; j < cl.size(); j++){
			
			
			if (!cl.get(j).verbID.equals("")) {
				
				return j;
			}
		}

		return -1;
	}
	
	public Verb getVerbForm (Clause cl){
		
		Verb v = new Verb();
		
		for (int j = 0; j < cl.size(); j++){
			
			
			if (!cl.get(j).verbID.equals("")) {
				
				v.add(cl.get(j));
				
				if (v.agreementObject.gender.equals("")) v.agreementObject.setGenderFromWord(cl.get(j));
				if (v.agreementObject.number.equals("")) v.agreementObject.setNumberFromWord(cl.get(j));
				if (v.agreementObject.person.equals("")) v.agreementObject.setPersonFromWord(cl.get(j));
			}
		}

		return v;
	}

	
	public ArrayList<NounPhraseWords> getNounPhrases (Clause cl){
		
		ArrayList<NounPhraseWords> list = new ArrayList<NounPhraseWords>();
		System.out.println(cl.getFullVerbForm().toString());
		int verbind = getVerbIndex(cl);
		
f1:		for (int j = 0; j < cl.size(); j++){
			
			Word w = cl.get(j);

			System.out.print(j + "\t" + verbind);
			if (j < verbind && w.pos.equals("U") && list.size()>0) {
				list = new ArrayList<NounPhraseWords>();
				
				System.out.println("IND:"+j);
					

				continue;
			}
			
				
			if (w.pos.startsWith("N") || w.reference!=null) {
				
				NounPhraseWords np = new NounPhraseWords();
				
				if(w.pos.startsWith("N")) np.add(w);
				else np.add(w.reference);
				
				np.head = w;
				np.headID = j;
				np.start = j;
				np.end = j;
				
				for (int k = j-1; k > 0; k--){
					if (cl.get(k).pos.startsWith("A")) {
						np.add(0, cl.get(k));
						np.start = k;
					}
					else break;
				}
				
				// 

				if (j < cl.size()-1 && cl.get(j+1).pos.startsWith("N")) {
					np.add(cl.get(j+1));
					np.end += 1;
					j++;
				}

				
				
				
			if (verbind < np.start || np.start == 0
						|| !cl.get(np.start-1).pos.equals("R")) {
					
				System.out.println("CL:" + cl.toString());
				System.out.println("NP:" +np.toString());
				System.out.println("?:" +(verbind));
				System.out.println("?:" +(np.start));

				if (verbind < np.start && list.size()>0) {
					list = new ArrayList<NounPhraseWords>();
					
				}
				list.add(np);
					
	
				}

			}
			
			
		}
		
		return list;
	}

	
	public String getVerbGender (Clause cl){
		
		
		for (int j = 0; j < cl.size(); j++){
			
			
			
			if (!cl.get(j).verbID.equals("")) {
				
				if (cl.get(j).pos.contains("m")) return "m";
				if (cl.get(j).pos.contains("f")) return "f";
				if (cl.get(j).pos.contains("n")) return "n";
			}
		}

		return "";
	}

	public String[] timeConj = {
			"преди да",
			"след като",
			"след което",
			"при което",
			"веднага щом",
			"едва когато",
			"още когато",
			"още като",
			"още щом",
			"веднъж да",
			"когато",
			"като",
			"докато",
			"щом"
				};

	public int countTimeConjunctions(Sentence sent){
		

		
		int ctr = 0;
		for (int j = 0; j < sent.size(); j++){
			
consf:		for (String con : timeConj) {
				
				String[] conparts = con.split(" ");
				
				if (j+conparts.length < sent.size()) {
					int ind = j;
					for (String part : conparts) {
						if (!sent.get(ind).lemma.equalsIgnoreCase(part)){
							continue consf;
						}
						ind++;
	
					}
					
					ctr++;
					j = ind;

				}
			}
		}
		
		return ctr;
		
		
	}
	


}
