package bg.bas.dcl.multilingual.ClauseSplitting;

import java.io.File;
import java.io.FileOutputStream;
import java.io.IOException;
import java.io.OutputStreamWriter;
import java.io.Writer;
import java.util.ArrayList;
import java.util.HashMap;
import java.util.List;
import java.util.Scanner;

import bg.bas.dcl.entities.Conjunction;
import bg.bas.dcl.entities.Sentence;
import bg.bas.dcl.entities.TextPosition;
import bg.bas.dcl.entities.Word;

import bg.bas.dcl.entities.VerbForm;
import bg.bas.dcl.general.FileHandler;

public class ClauseSplittingFileProcessor {

	
	
	public void printOutResults(Sentence sent, 
			ArrayList<VerbForm> verbs){
		
		
		System.out.println(verbs.size());
		
		for (VerbForm form : verbs){
			System.out.print("FORM: ");
			
			for (TextPosition i : form) {
				System.out.print(sent.get(i.sentpos).wordform + " ");
			}
//			System.out.println("," + form.tense);
		}
		
		System.out.println("***");

	}
	
	public void writeResults(Sentence sent, 
			ArrayList<VerbForm> verbs,
			Writer out) throws IOException{

		for (VerbForm form : verbs){
			
			out.write("FORM: ");
			for (TextPosition i : form) {
				out.write(sent.get(i.sentpos).wordform + " ");
			}

//			out.write("," + form.tense+ "\n");
		}
		
		out.write("\n\n***\n\n");
		out.flush();
		
	}
	
	public void writeResultsHtml(Sentence sent, 
			ArrayList<VerbForm> verbs,
			Writer out) throws IOException{

		String[] colours = {
				"#0066FF",
				"#FF3300",
				"#999999",
				"#FFCC00",
				"#99CC00",
				"#CC00CC",
				"#009933",
				"#FF33CC",
				"#3300CC",
				"#CC0033",
				"#333300"
				};

		int pos = 0;
		for (int i=0; i<verbs.size(); i++){
			
			VerbForm form = verbs.get(i);
			
			int i1 = form.get(0).sentpos;
			int i2 = form.get(form.size()-1).sentpos;
			
			for (int j=pos; j<i1; j++) {
				out.write(sent.get(j).wordform+","+sent.get(j).pos + " ");
			}

			for (int j=i1; j<i2+1; j++) {
				if (form.contains(j)) {
					out.write(
							"<font color=\""
							+ colours[i % colours.length]+"\"><b>"
							+ sent.get(j).wordform+","+sent.get(j).pos + "</b></font> ");
				} else {
					out.write(sent.get(j).wordform+","+sent.get(j).pos + " ");
				}
			}
			
			pos = i2+1;

//			out.write(
	//				"<font color=\""
//							+ colours[i % colours.length]+"\">"
//							+ form.tense + "</font> ");
		}
		
		for (int j=pos; j<sent.size(); j++) {
			out.write(sent.get(j).wordform + " ");
		}

		out.write("<br><br>***<br><br>");
		out.flush();
		
	}

	
	public Sentence readSentence(Scanner s){
		
		Sentence sent = new Sentence();
		
		while (s.hasNextLine()) {
			String l = s.nextLine();
			
			if (l.startsWith("</S>")) return sent;
			
			Word t = new Word();
			t.readFromVertical(l);
			
			sent.add(t);
		}
		
		return sent;
	}
	

	public ArrayList<Conjunction> readConjunctionsFromFile(String filename){
		ArrayList<Conjunction> result = new ArrayList<Conjunction>();
		
		try{
			Scanner s = new Scanner(new File(filename));
			
			while(s.hasNextLine()){
				String l = s.nextLine();
				
				if (l.trim().equals("")) continue;
				String[] data = l.split("\t");
				
				if (data.length<5) continue;
				
				String[] lemmas = data[0].split(";");
				
				
ll:				for (String lemma : lemmas){
					
					Conjunction c = new Conjunction();
					c.lemma = lemma.trim().toLowerCase();
					if (c.lemma.equals("")) continue;
					
					c.type = data[1];
					c.isCompulsaryCO = (data[2].equalsIgnoreCase("true")? true : false);
					c.preceedingCommaCO = (data[3].equalsIgnoreCase("true")? true : false);
					c.semanticType = data[4];
					
					if (result.size()==0) result.add(c);
					
					for (int i = 0; i<result.size(); i++){
	
						if (result.get(i).lemma.split(" ").length <= c.lemma.split(" ").length){
							
							System.out.println(c.lemma + "\t" + c.type + "\t" 
							+ c.isCompulsaryCO + "\t"+ c.preceedingCommaCO + "\t" + c.semanticType);
						
							result.add(i, c);
							continue ll;
	
						}
					}
					
					result.add(c);
				}

			}
			
			s.close();
		
		} catch(Exception e){
			e.printStackTrace();
		}
		
		return result;
	}
	
	public HashMap<String,ArrayList<String>> readSpecialVerbsFromFile(String filename){
		HashMap<String,ArrayList<String>> result = new HashMap<String,ArrayList<String>>();
		
		try{
			Scanner s = new Scanner(new File(filename));
			
			while(s.hasNextLine()){
				String l = s.nextLine();
				
				if (l.trim().equals("")) continue;
				String[] data = l.split("\t");
				
				if (data.length<3) continue;
	
				ArrayList<String> list = new ArrayList<String>();
				if (result.containsKey(data[2].trim())) list = result.get(data[2].trim());
				
				if (!data[0].trim().equals("") && !list.contains(data[0].trim())) list.add(data[0].trim());

				result.put(data[2].trim(), list);
			}
			
			s.close();
		
		} catch(Exception e){
			e.printStackTrace();
		}
		
		return result;
	}

}
