package bg.bas.dcl.multilingual.ClauseSplitting;

import java.io.File;
import java.io.FileNotFoundException;
import java.io.FileOutputStream;
import java.io.IOException;
import java.io.OutputStreamWriter;
import java.io.UnsupportedEncodingException;
import java.io.Writer;
import java.util.ArrayList;
import java.util.Collection;
import java.util.HashMap;
import java.util.List;
import java.util.Scanner;

import javax.xml.parsers.DocumentBuilder;
import javax.xml.parsers.DocumentBuilderFactory;

import org.w3c.dom.Document;
import org.w3c.dom.Element;
import org.w3c.dom.Node;
import org.w3c.dom.NodeList;

import bg.bas.dcl.entities.ClauseConjunction;
import bg.bas.dcl.entities.Conjunction;
import bg.bas.dcl.entities.Sentence;
import bg.bas.dcl.entities.VerbForm;
import bg.bas.dcl.entities.Word;
import bg.bas.dcl.general.FileHandler;
import bg.bas.dcl.monolingual.bg.TextProcessor;
import bg.bas.dcl.multilingual.Chunker.VerbFormRecogniser.ClauseVerb;
import bg.bas.dcl.multilingual.Chunker.VerbFormRecogniser.ClauseVerbsVerifier;
import bg.bas.dcl.multilingual.Chunker.VerbFormRecogniser.VerbFormFormalism;
import bg.bas.dcl.multilingual.ClauseSplitting.rules.ArbitraryClauseSplittingRule;
import bg.bas.dcl.multilingual.ClauseSplitting.rules.BeginSentenceClauseSplittingRule;
import bg.bas.dcl.multilingual.ClauseSplitting.rules.ClauseSplittingRule;
import bg.bas.dcl.multilingual.ClauseSplitting.rules.ClosingCommaClauseSplittingRule;
import bg.bas.dcl.multilingual.ClauseSplitting.rules.ConjunctionClauseSplittingRule;
import bg.bas.dcl.multilingual.ClauseSplitting.rules.EndSentenceClauseSplittingRule;
import bg.bas.dcl.multilingual.ClauseSplitting.rules.OptionalConjunctionClauseSplittingRule;
import bg.bas.dcl.multilingual.ClauseSplitting.rules.OptionalConjunctionsAfterClauseSplittingRule;
import bg.bas.dcl.multilingual.ClauseSplitting.rules.ParenthericalsClauseSplittingRule;
import bg.bas.dcl.multilingual.ClauseSplitting.rules.PunctuationClauseSplittingRule;
import bg.bas.dcl.multilingual.ClauseSplitting.rules.StrongPunctuationClauseSplittingRule;


public class RuleBasedClauseSplitter {
	
	String[] colours = {
			"#0066FF",
			"#FF3300",
			"#999999",
			"#FFCC00",
			"#99CC00",
			"#CC00CC",
			"#009933",
			"#FF33CC",
			"#3300CC",
			"#CC0033",
			"#555500"
			};

	ArrayList<ClauseSplittingRule> rules = new ArrayList<ClauseSplittingRule>();
	ArrayList<Conjunction> conjunctions = new ArrayList<Conjunction>();
	
	ClauseSplittingFileProcessor csfp = new ClauseSplittingFileProcessor();
	
	ClauseVerbsVerifier cvv;
	
	VerbFormFormalism vff;
	
	public RuleBasedClauseSplitter(VerbFormFormalism vff, 
			String conjFileName,
			String verbsFileName,
			String punctExcFile){

		this.vff = vff;
		
		this.cvv = new ClauseVerbsVerifier(verbsFileName);
		
		conjunctions = csfp.readConjunctionsFromFile(conjFileName);

/**
 		rules.add(new BeginSentenceClauseSplittingRule());

		rules.add(new StrongPunctuationClauseSplittingRule(punctExcFile));
		
		rules.add(new ParenthericalsClauseSplittingRule());
		rules.add(new ConjunctionClauseSplittingRule(conjunctions));
		rules.add(new ClosingCommaClauseSplittingRule(false));

		rules.add(new OptionalConjunctionClauseSplittingRule(conjunctions));

		rules.add(new PunctuationClauseSplittingRule(false));
//		rules.add(new OptionalConjunctionsAfterClauseSplittingRule(conjunctions));
		
		rules.add(new ArbitraryClauseSplittingRule());
		rules.add(new EndSentenceClauseSplittingRule());
		*/
		
		rules.add(new BeginSentenceClauseSplittingRule());
		rules.add(new StrongPunctuationClauseSplittingRule(punctExcFile));
		
//		rules.add(new ParenthericalsClauseSplittingRule());
		rules.add(new ConjunctionClauseSplittingRule(conjunctions));
		rules.add(new ClosingCommaClauseSplittingRule(true));

		rules.add(new OptionalConjunctionClauseSplittingRule(conjunctions));
//		rules.add(new ClosingCommaClauseSplittingRule(false));

		rules.add(new PunctuationClauseSplittingRule(false));
//		rules.add(new OptionalConjunctionsAfterClauseSplittingRule(conjunctions));
		
		rules.add(new ArbitraryClauseSplittingRule());
		rules.add(new EndSentenceClauseSplittingRule());

	}
	
	public String getSentencePlainText(Sentence sent){
		String result = "";
		
		for (Word t : sent) result += t.wordform + " ";
		
		return result;
	}
	
	
	public void processCorpus(String corpusDir, String outfile){
		
		FileHandler fh = new FileHandler();

		try{
			Writer out = new OutputStreamWriter(
					new FileOutputStream(outfile+".html"));

			
			List<File> list = fh.getFileListing(new File(corpusDir));
			
			for (File f : list) {
				
				if (!f.isFile()) continue;
				if (f.getName().startsWith(".")) continue;
				if (f.getName().contains("~")) continue;
				
				processFile(f, out);

//				System.exit(0);
			}

			out.flush();
			out.close();
		} catch(Exception e){
			e.printStackTrace();
		}
		
	}

	public void processCorpusXML(String corpusDir, String outfile, String ext){
		
		FileHandler fh = new FileHandler();

		try{
			Writer out = new OutputStreamWriter(
					new FileOutputStream(outfile+".html"));

			Writer errorout = new OutputStreamWriter(
					new FileOutputStream(outfile+".error.html"));

			
			for (int i = 0; i<22; i++) evaluationResults.add(0);
			

			List<File> list = fh.getFileListing(new File(corpusDir));
			
			for (File f : list) {
				
				if (!f.isFile()) continue;
				if (f.getName().startsWith(".")) continue;
				if (!f.getName().endsWith(ext)) continue;
				if (f.getName().contains("~")) continue;
				
				processFileXML(f, out, errorout);

//				break;
//				System.exit(0);
			}

			out.flush();
			out.close();
			
			errorout.flush();
			errorout.close();
			
			System.out.println(printEvaluationResults());
			
		} catch(Exception e){
			e.printStackTrace();
		}
		
	}
	
	public void processCorpusWithTagging(
			String corpusDir, String outfile, String ext){
		
		FileHandler fh = new FileHandler();

		try{
			Writer out = new OutputStreamWriter(
					new FileOutputStream(outfile+".html"));

			Writer errorout = new OutputStreamWriter(
					new FileOutputStream(outfile+".error.html"));

			
			for (int i = 0; i<22; i++) evaluationResults.add(0);
			

			List<File> list = fh.getFileListing(new File(corpusDir));
			
			for (File f : list) {
				
				if (!f.isFile()) continue;
				if (f.getName().startsWith(".")) continue;
				if (!f.getName().endsWith(ext)) continue;
				if (f.getName().contains("~")) continue;
				
				processFileWithTagging(f, out, errorout);

//				break;
//				System.exit(0);
			}

			out.flush();
			out.close();
			
			errorout.flush();
			errorout.close();
			
			System.out.println(printEvaluationResults());
			
		} catch(Exception e){
			e.printStackTrace();
		}
		
	}
	
	public ArrayList<ClauseVerb> getClauseVerbs (Sentence sent){
		
		ArrayList<VerbForm> verbs1 = vff.findForms(sent);
		ArrayList<ClauseVerb> verbs = new ArrayList<ClauseVerb>();
		
		verbs.addAll(cvv.reviseVerbs(sent, verbs1));
		
		return verbs;

	}
	public Sentence performClauseSplitting (Sentence sent){
		
		ArrayList<VerbForm> verbs1 = vff.findForms(sent);
		ArrayList<ClauseVerb> verbs = new ArrayList<ClauseVerb>();
		
		verbs.addAll(cvv.reviseVerbs(sent, verbs1));
		
		ClauseSplittingComplexObject obj = processSentence(sent, verbs);
		sent = assignClauses(sent, obj.clborders);
		
		return sent;
	}

	public void processFile(File f, Writer out){
		
		try{
			Scanner s = new Scanner(f);
			
			out.write(f.getAbsolutePath()+"<br><br>");
			
			while (s.hasNextLine()){
			
				String line = s.nextLine();
				if (!line.startsWith("<S")) continue;
				
				Sentence sent = new Sentence();
				sent = csfp.readSentence(s);
				
				ArrayList<VerbForm> verbs1 = vff.findForms(sent);
				ArrayList<ClauseVerb> verbs = new ArrayList<ClauseVerb>();
				
				verbs.addAll(cvv.reviseVerbs(sent, verbs1));
				
				ClauseSplittingComplexObject obj = processSentence(sent, verbs);
//				ArrayList<ClauseBorder> allboundaries = obj.clborders;
				sent = assignClauses(sent, obj.clborders);			
				
				out.write(verbs.size()+"\t"+obj.clborders.size()+"<br>");
				printColourHTML(out, sent, obj, verbs);
				
//				System.out.println(getSentencePlainText(sent));
//				System.out.println("SPLITTING:");
				
				for (ClauseBorder cb : obj.clborders) {
					
					System.out.println(cb.position+"\t"+cb.type);
				}
				
				System.out.println("*****");
				
				out.write("\n");
				out.flush();
				 
			
			}
			s.close();
		} catch(Exception e){
			e.printStackTrace();
		}
		
	}
	
	ArrayList<Integer> evaluationResults = new ArrayList<Integer>();
	
	public void updateEvaluationResults(ArrayList<Integer> res) {
		
		for (int i=0; i<Math.min(res.size(),evaluationResults.size());i++){
			evaluationResults.set(i, evaluationResults.get(i)+res.get(i));
		}
	}
	
	
	public String printEvaluationResults() {
		
		String res = "";

		res += 
				"PREC CLAUSES" + "\t" + evaluationResults.get(cse.CORRECTCLAUSE)
					+"/"+evaluationResults.get(cse.CLAUSE) + "\t" +
					1.0*evaluationResults.get(cse.CORRECTCLAUSE)/evaluationResults.get(cse.CLAUSE)
					+"\n"
				+ "PREC BORDERS" + "\t" + evaluationResults.get(cse.CORRECTBORDER)
					+"/"+evaluationResults.get(cse.BORDER) + "\t" +
					1.0*evaluationResults.get(cse.CORRECTBORDER)/evaluationResults.get(cse.BORDER)
					+"\n"
				+ "PREC CO" + "\t" + evaluationResults.get(cse.CORRECTOPENING)
					+"/"+evaluationResults.get(cse.OPENING) + "\t" +
					1.0*evaluationResults.get(cse.CORRECTOPENING)/evaluationResults.get(cse.OPENING)
					+"\n"
				+ "PREC CC" + "\t" + evaluationResults.get(cse.CORRECTCLOSING)
					+"/"+evaluationResults.get(cse.CLOSING) + "\t" +
					1.0*evaluationResults.get(cse.CORRECTCLOSING)/evaluationResults.get(cse.CLOSING)
					+"\n"
				+ "PREC SENT" + "\t" + evaluationResults.get(cse.CORRECTSENTENCE)
					+"/"+evaluationResults.get(cse.SENTENCE) + "\t" +
					1.0*evaluationResults.get(cse.CORRECTSENTENCE)/evaluationResults.get(cse.SENTENCE)
					+"\n"
				+ "PREC CONJ" +"\t" + correctC + "/" + allC + "\t" + 1.0*correctC/allC
					+"\n"
				+ "PREC CONJ REL" +"\t" + correctCR + "/" + allC + "\t" + 1.0*correctCR/allC
					+"\n"
				+ "PREC CONJ REL+REF" +"\t" + correctCRR + "/" + allC + "\t" + 1.0*correctCRR/allC
					+"\n* * *\n"
				+ "RECALL BORDERS" + "\t" + evaluationResults.get(cse.CORRECTBORDER)
					+"/"+evaluationResults.get(cse.REALBORDER) + "\t" +
					1.0*evaluationResults.get(cse.CORRECTBORDER)/evaluationResults.get(cse.REALBORDER)
					+"\n"
				+ "RECALL CO" + "\t" + evaluationResults.get(cse.CORRECTOPENING)
					+"/"+evaluationResults.get(cse.REALOPENING) + "\t" +
					1.0*evaluationResults.get(cse.CORRECTOPENING)/evaluationResults.get(cse.REALOPENING)
					+"\n"
				+ "RECALL CC" + "\t" + evaluationResults.get(cse.CORRECTCLOSING)
					+"/"+evaluationResults.get(cse.REALCLOSING) + "\t" +
					1.0*evaluationResults.get(cse.CORRECTCLOSING)/evaluationResults.get(cse.REALCLOSING)
				;

		return res;
	
	}
	
	public Sentence processSentence(Sentence sent){
		
		ArrayList<VerbForm> verbs1 = vff.findForms(sent);
		ArrayList<ClauseVerb> verbs = cvv.reviseVerbs(sent, verbs1);	

		ClauseSplittingComplexObject obj = processSentence(sent, verbs);
		
//		System.out.println("BOUNDARIES "+obj.clborders.size());

		sent = assignClauses(sent, obj.clborders);
		
		obj.clborders = reviseBoundaries(sent, obj.clborders);

//		System.out.println("BOUNDARIES "+obj.clborders.size());

		sent = assignClauses(sent, obj.clborders);

		return sent;
	}

	
	public void processFileXML(File f, Writer out, Writer errorout){
		
		try{

			DocumentBuilderFactory dbFactory = DocumentBuilderFactory.newInstance();
			DocumentBuilder dBuilder = dbFactory.newDocumentBuilder();
			Document doc = dBuilder.parse(f);
			 
			//optional, but recommended
			//read this - http://stackoverflow.com/questions/13786607/normalization-in-dom-parsing-with-java-how-does-it-work
			doc.getDocumentElement().normalize();
			
			out.write(f.getAbsolutePath()+"<br><br>");
			errorout.write(f.getAbsolutePath()+"<br><br>");
			
			System.out.println("File: "+f.getAbsolutePath());
 
			Sentence sent = new Sentence();
			
			
			NodeList sentList = doc.getElementsByTagName("S");

			
			for (int j = 0; j < sentList.getLength(); j++) {
				
				Node sentNode = sentList.item(j);
				
				if (sentNode.getNodeType() != Node.ELEMENT_NODE) continue;
				
				Element sentElement = (Element) sentNode;
				
				NodeList wordList = sentElement.getElementsByTagName("word");
				
				sent = new Sentence();
								
				for (int i = 0; i < wordList.getLength(); i++) {
					 
					Node wordNode = wordList.item(i);
					
					if (wordNode.getNodeType() != Node.ELEMENT_NODE) continue;
					
					Element wordElement = (Element) wordNode;
				
//					System.out.println("WORD: "+wordElement.getAttribute("w")
//							+" "+wordElement.getAttribute("pos"));
	
					Word t = new Word();
					t.wordform = wordElement.getAttribute("w");
					
					if (t.wordform.equals("====")) continue;
						
					t.pos = wordElement.getAttribute("pos");
					t.lemma = wordElement.getAttribute("l");
					t.remainingAttr.put("old_clauseID", 
							wordElement.getAttribute("cl"));
					
					if (wordElement.hasAttribute("p")){
						t.MWE_part = wordElement.getAttribute("p");
					}
					if (wordElement.hasAttribute("m")){
						t.remainingAttr.put("relation",
								wordElement.getAttribute("m"));
					}
					if (wordElement.hasAttribute("cl2")){
						t.remainingAttr.put("cl2",
								wordElement.getAttribute("cl2"));
					}
						
					if (t.pos.equals("")) t.pos="M";
					
					sent.add(t);
	
				}
								
				ArrayList<VerbForm> verbs1 = vff.findForms(sent);
				
				ArrayList<ClauseVerb> verbs = cvv.reviseVerbs(sent, verbs1);	

//				System.out.println("SEN CHECK " 
//						+ verbs1.size() + "\t" + verbs.size());
				
//				System.out.print("VERBS1 " );
//				for(VerbForm vf : verbs1) System.out.print(sent.get(vf.get(0)).wordform+" ");
//				System.out.println();
								
				ClauseSplittingComplexObject obj = processSentence(sent, verbs);
				
//				System.out.println("BOUNDARIES "+obj.clborders.size());

				sent = assignClauses(sent, obj.clborders);
				
				obj.clborders = reviseBoundaries(sent, obj.clborders);

//				System.out.println("BOUNDARIES "+obj.clborders.size());

				sent = assignClauses(sent, obj.clborders);

				
				ArrayList<Integer> res = evaluate(sent, obj, verbs, errorout);
				updateEvaluationResults(res);
				
				out.write(verbs.size()+"\t"+obj.clborders.size()+"<br>");
				printColourHTML(out, sent, obj, verbs);
				
//				System.in.read();
				
//				System.out.println(getSentencePlainText(sent));
//				System.out.println("SPLITTING:");
				
				for (ClauseBorder cb : obj.clborders) {
					
//					System.out.println(cb.position+"\t"+cb.type);
				}
				
				System.out.println("*****");
				
				out.write("\n");
				out.flush();

				

				
			}

			
		} catch(Exception e){
			e.printStackTrace();
			System.exit(0);
		}
		
	}
	
	TextProcessor tp = new TextProcessor();

	public void processFileWithTagging(File f, Writer out, Writer errorout){
		
		
		try{


			out.write(f.getAbsolutePath()+"<br><br>");
			errorout.write(f.getAbsolutePath()+"<br><br>");
			
			System.out.println("File: "+f.getAbsolutePath());
 
			Sentence sent = new Sentence();
			
			Scanner s = new Scanner(f);
			
			
			while (s.hasNextLine()) {
				
				String line = s.nextLine();
				
				String tagged = tp.applyServiceToText(line, 2);
				
				System.out.println(tagged);
				
				String[] words = tagged
						.split("\n");
	
				sent = new Sentence();
								
				for (String w : words) {
					 
					String[] data = w.split("\t");
					
					Word t = new Word();
					t.wordform = data[0];
					t.pos = data[data.length-1];
					t.lemma = data[data.length-2];
					t.remainingAttr.put("old_clauseID", 
							"0");

						
					if (t.pos.equals("")) t.pos="M";
					
					sent.add(t);
	
				}
								
				ArrayList<VerbForm> verbs1 = vff.findForms(sent);
				
				ArrayList<ClauseVerb> verbs = cvv.reviseVerbs(sent, verbs1);	

//				System.out.println("SEN CHECK " 
//						+ verbs1.size() + "\t" + verbs.size());
				
//				System.out.print("VERBS1 " );
//				for(VerbForm vf : verbs1) System.out.print(sent.get(vf.get(0)).wordform+" ");
//				System.out.println();
								
				ClauseSplittingComplexObject obj = processSentence(sent, verbs);
				
//				System.out.println("BOUNDARIES "+obj.clborders.size());

				sent = assignClauses(sent, obj.clborders);
				
				obj.clborders = reviseBoundaries(sent, obj.clborders);

//				System.out.println("BOUNDARIES "+obj.clborders.size());

				sent = assignClauses(sent, obj.clborders);

				
				ArrayList<Integer> res = evaluate(sent, obj, verbs, errorout);
				updateEvaluationResults(res);
				
				out.write(verbs.size()+"\t"+obj.clborders.size()+"<br>");
				printColourHTML(out, sent, obj, verbs);
				
//				System.in.read();
				
//				System.out.println(getSentencePlainText(sent));
//				System.out.println("SPLITTING:");
				
				for (ClauseBorder cb : obj.clborders) {
					
//					System.out.println(cb.position+"\t"+cb.type);
				}
				
				System.out.println("*****");
				
				out.write("\n");
				out.flush();

				

				
			}

			s.close();
			
		} catch(Exception e){
			e.printStackTrace();
			System.exit(0);
		}
		
	}

	public boolean isSubordinateConjunction(Word t){
		
		for (Conjunction c : conjunctions) {
			if (c.lemma.startsWith(t.lemma) 
					&& c.type.startsWith("Sub")) return true;
		}
		return false;
	}
	
	
	public ArrayList<ClauseBorder> reviseBoundaries(Sentence sent, 
			ArrayList<ClauseBorder> allboundaries){

		ArrayList<ClauseBorder> result = new ArrayList<ClauseBorder>();
		ArrayList<Integer> ignore = new ArrayList<Integer>();
		
		sent = assignClauses(sent, allboundaries);
		
		for (int i = 0; i < allboundaries.size(); i++){
			
			ClauseBorder boundary = allboundaries.get(i);
			
			if (boundary.position >= sent.size()) {
//				System.out.println(boundary.position+","+boundary.type+" " + sent.size());
//				System.exit(0);
			}
			
			if (boundary.type.contains("CC")){
				if(!ignore.contains(i)) {
					result.add(boundary);	
				} 
				
				continue;
			}
			
			if (boundary.type.contains("CO")){
				
				/**
					if (allboundaries.size()>0 
						&& allboundaries.get(i-1).position==evboundary.position
						&& allboundaries.get(i-1).type.contains("COC")){
						
						
					}
				*/

					while (boundary.position > 0
							&& ((sent.get(boundary.position-1).pos.equals("R")
										&& isSubordinateConjunction(sent.get(boundary.position)))
									|| sent.get(boundary.position-1).pos.equals("C"))){
						
						boundary.position = boundary.position-1;
						
						int k=i-1;
						while (k>=0 && allboundaries.get(k).position>=boundary.position){
							allboundaries.get(k).position = 
									Math.max(0,allboundaries.get(k+1).position-1);
							k--;
						}
					}
			}
			
			result.add(boundary);
			
/**
			String clID = sent.get(
					boundary.position)
					.clauseID;
//			System.out.println("Looking for "+clID);
			
//			result.add(boundary);
			/**
			boolean isClause = false;
			for (int j = 0; j < sent.size(); j++){
				
				System.out.print(sent.get(j).clauseID+" ");
				
				if (clID.equals(sent.get(j).clauseID)
						&& !sent.get(j).pos.startsWith("R")
						&& !sent.get(j).pos.startsWith("D")
						&& !sent.get(j).pos.startsWith("I")
						&& !sent.get(j).pos.startsWith("U")
						&& !sent.get(j).pos.startsWith("T")
						&& !sent.get(j).pos.startsWith("C")){
					isClause = true;
					break;
				}
			}
			
			System.out.println(isClause);
			
			/**
			
			
			if (isClause) {
				result.add(boundary);	
			} else {
				
				System.out.println("REMOVE BORDER "+i);
				int t = 0;
				int k = i+1;
				
				for (k = i+1; k < allboundaries.size(); k++){
					if (allboundaries.get(k).type.contains("CC") && t==0) {
						break;
					} else {
						if  (allboundaries.get(k).type.contains("CC")) t--;
						else t++;
					}
				}
				
				System.out.println("REMOVE BORDER CC "+(k));
				ignore.add(k);
				
				if (k>i+1){
					allboundaries.get(i+1).position = allboundaries.get(i).position;
					allboundaries.get(k-1).position = allboundaries.get(k).position;
				}
			}
			*/
			
		}
		
		return result;
	}
	
	ClauseSplitterEvaluator cse = new ClauseSplitterEvaluator();
	
	public ArrayList<Integer> evaluate(Sentence sent, 
			ClauseSplittingComplexObject obj,
			ArrayList<ClauseVerb> verbs, 
			Writer errorout){
		
//		ArrayList<Integer> result = cse.evaluateClauseSplittingSentence(sent, 
//				allboundaries, verbs);
		ArrayList<Integer> result = cse.evaluateClauseSplittingSentence(sent, verbs);
		

		try {
			

			if(result.get(cse.CORRECTCLAUSE)<result.get(cse.CLAUSE)
					|| result.get(cse.CORRECTBORDER)<result.get(cse.REALBORDER)
					){

				printColourHTML(errorout, sent);				
				errorout.write("<br><br>");			
				printColourHTML(errorout, sent, obj, verbs);
				errorout.write("<br><br>");			
				
				errorout.write(
						"PRECISION " + result.get(cse.CORRECTCLAUSE) + "/" + result.get(cse.CLAUSE) + "<br>"
						+ "PRECIS CO " + result.get(cse.CORRECTOPENING) + "/" + result.get(cse.OPENING) + "<br>"
						+ "PRECIS CC " + result.get(cse.CORRECTCLOSING) + "/" + result.get(cse.CLOSING) + "<br>"
						+ "PRECIS  B " + result.get(cse.CORRECTBORDER) + "/" + result.get(cse.BORDER) + "<br>"
						);
				errorout.write("*****<br><br>");
				errorout.flush();
				
				System.out.println(
						"PRECISION " + result.get(cse.CORRECTCLAUSE) + "/" + result.get(cse.CLAUSE) + "\n"
						+ "PRECIS CO " + result.get(cse.CORRECTOPENING) + "/" + result.get(cse.OPENING) + "\n"
						+ "PRECIS CC " + result.get(cse.CORRECTCLOSING) + "/" + result.get(cse.CLOSING) + "\n"
						+ "PRECIS  B " + result.get(cse.CORRECTBORDER) + "/" + result.get(cse.BORDER) + "\n"
						);


			}
		} catch (Exception e) {
			// TODO Auto-generated catch block
			e.printStackTrace();
		}
		
		return result;
		
	}
	
	int ctr = 0;
	public void printColourHTML(Writer out, Sentence sent, 
			ClauseSplittingComplexObject obj,
			ArrayList<ClauseVerb> verbs){
		
		String result = "";
		int vi=0;
		ClauseVerb vf = new ClauseVerb();
				
		if (vi < verbs.size()) vf=verbs.get(vi);
		
		int ci = 0;
		int di = 0;
		ClauseBorder cb = obj.clborders.get(ci);
		
		String borders = "";
		
		for (int i = 0; i<sent.size(); i++){
			Word t = sent.get(i);
		
			if(i==cb.position){
				if (cb.type.contains("CO")) {
					result += "<font color=\"" 
							+ colours[ctr++ % colours.length] + "\">";
				}
				
			}
			

			if(vf.contains(i)) {
				result += "<b>";
			}
			
			if(di<obj.clconjunctions.size() 
					&& obj.clconjunctions.get(di).contains(i)) {
					result += "<u>";
			}
			

			
			result += t.wordform+"/"+t.pos;
			
			if(vf.contains(i)) {
				result += "</b>";
				if(i==vf.get(vf.size()-1).sentpos) {
					vi++;
					if (vi<verbs.size()) vf=verbs.get(vi);
				}

			}
			
			if(di<obj.clconjunctions.size() 
					&& obj.clconjunctions.get(di).contains(i)) {
				result += "/"+t.remainingAttr.get("relation")+"</u>";
				
				if(i==obj.clconjunctions.get(di)
						.get(obj.clconjunctions.get(di).size()-1)) {
					di++;
				}

			}
			
			result += " ";

			
			while(i==cb.position){
				
				borders += cb.type + "," +cb.position + " ";
				
				if(cb.type.startsWith("CC")) {

					result += "</font> ";
				}
				ci++;
				
				if (ci<obj.clborders.size()) {
					cb = obj.clborders.get(ci);
				} else {
					break;
				}
			}


			

		}
		
		
		
		try {
			out.write(result+"<br>"+borders+"<br><br>");
			out.flush();
			
		} catch (IOException e) {
			// TODO Auto-generated catch block
			e.printStackTrace();
		}
		
	}

	public void printColourHTML(Writer out, Sentence sent){
		
		String result = "";
				
		ArrayList<String> clIDs = new ArrayList<String>();

		
		for (int i = 0; i<sent.size(); i++){
			Word t = sent.get(i);
		
			if (!clIDs.contains(t.remainingAttr.get("old_clauseID"))) {
				clIDs.add(t.remainingAttr.get("old_clauseID"));
			}
			else {
//				System.out.println("CLID: "+t.old_clauseID);
			}

			result += "<font color=\"" 
							+ colours[clIDs.indexOf(t.remainingAttr.get("old_clauseID"))
							          % colours.length] + "\">";

			result += t.wordform+"/"+t.pos+" ";
			
			result += "</font> ";

		}
		
		
		try {
			out.write(result+"<br><br>");
			out.flush();
			
		} catch (IOException e) {
			// TODO Auto-generated catch block
			e.printStackTrace();
		}
		
	}

	
	ArrayList<Integer> errorCtr = new ArrayList<Integer>();
	ArrayList<Integer> total = new ArrayList<Integer>();
	
	public ClauseSplittingComplexObject processSentence(Sentence sent, 
			ArrayList<ClauseVerb> verbs){
		
		System.out.println("VERBS size "+verbs.size());
		
		ClauseSplittingComplexObject resultobj = new ClauseSplittingComplexObject();
		
//		ArrayList<ClauseBorder> allboundaries = new ArrayList<ClauseBorder>();
//		ArrayList<ClauseConjunction> allconjunctions = new ArrayList<ClauseConjunction>();
		
		ClauseStack cls = new ClauseStack();
		
		
		if (verbs.size() == 0) {
			resultobj.clborders.add(new ClauseBorder(0, "COC"));
			resultobj.clborders.add(new ClauseBorder(sent.size()-1, "CC"));
			
			return resultobj;
		}
		
		for (ClauseSplittingRule r : rules){
			if (r instanceof BeginSentenceClauseSplittingRule
					|| r instanceof ConjunctionClauseSplittingRule
					|| r instanceof StrongPunctuationClauseSplittingRule
					){
				
				ClauseSplittingComplexObject obj = r.findClauseBorders(
						sent, verbs, 0, verbs.get(0).get(0).sentpos, cls);
				resultobj.clborders.addAll(obj.clborders);
				resultobj.clconjunctions.addAll(obj.clconjunctions);
				
				cls = obj.cls;
				
				sent = assignClauses(sent, resultobj.clborders);
//				for (Token to : sent) System.out.print(to.wordform + "/" + to.clauseID+ " ");
//				System.out.println();
				
//				System.out.println("START: "+ allboundaries.size()+"\t"+cls.size());
				
			}
		}
		
		
		for (int i=0; i<verbs.size()-1; i++){
			

			int j1 = verbs.get(i).get(verbs.get(i).size()-1).sentpos;
			int j2 = verbs.get(i+1).get(0).sentpos-1;

//			System.out.println("VERB LEN " + verbs.get(i).size() + "\t" + verbs.get(i+1).size());
//			System.out.println("INDICES: "+j1+"\t"+j2);

			if (j1 >= j2) continue;
		
						
			for (ClauseSplittingRule r : rules){
				
				if (r instanceof BeginSentenceClauseSplittingRule) continue;
				if (r instanceof EndSentenceClauseSplittingRule) continue;
				
//				System.out.println(r.getClass() + " begin STACK " + cls.stackSize());

				
				ClauseSplittingComplexObject obj = r.findClauseBorders(
						sent,
						verbs, 
						j1, j2,
						cls
						);

				resultobj.clborders.addAll(obj.clborders);
				resultobj.clconjunctions.addAll(obj.clconjunctions);

				cls = obj.cls;
				
				sent = assignClauses(sent, resultobj.clborders);

//				System.out.println(r.getClass()+" "
//						+ sent.get(verbs.get(i).get(0).sentpos).wordform
//						+" "+sent.get(verbs.get(i+1).get(0).sentpos).wordform);
//				for (Token to : sent) System.out.print(to.wordform + "/" + to.clauseID+ " ");
//				System.out.println();

				if (
						obj.clborders.size()>0 
						&& 
						r.isEnding()) {
//						System.out.println("ENDING RULE " + r.getClass());
						
						for (ClauseBorder cb : resultobj.clborders){
//							System.out.print(cb.type + "," + cb.position + " ");
						}
						
						System.out.println();

						break;
				}
				
//				System.out.println(r.getClass() + " end STACK " + cls.stackSize());


			}
			
		}

		for (ClauseSplittingRule r : rules){
			if (r instanceof EndSentenceClauseSplittingRule
					|| r instanceof ConjunctionClauseSplittingRule
//					|| r instanceof StrongPunctuationClauseSplittingRule
					){
//				System.err.println(verbs.get(verbs.size()-1).get(verbs.get(verbs.size()-1).size()-1)
//						+ (sent.size()-1));
				
				ClauseSplittingComplexObject obj = r.findClauseBorders(
						sent, 
						verbs, 
						verbs.get(verbs.size()-1).get(verbs.get(verbs.size()-1).size()-1).sentpos+1, 
						sent.size()-1 ,cls);

						resultobj.clborders.addAll(obj.clborders);
//						cls.clear();
						cls = obj.cls;
						
						resultobj.clconjunctions.addAll(obj.clconjunctions);

						
						sent = assignClauses(sent, resultobj.clborders);
//						for (Token to : sent) System.out.print(to.wordform + "/" + to.clauseID+ " ");
//						System.out.println();
						
//				break;
			}
		}

		
		evaluateConjunctions(sent, resultobj.clconjunctions);
		return resultobj;
		
	}
	
	int realC = 0;
	int allC = 0;
	int correctC = 0;
	int correctCR = 0;
	int correctCRR = 0;

	public void evaluateConjunctions(Sentence sent, ArrayList<ClauseConjunction> conj){
		
		for (ClauseConjunction clc : conj) {
			
			realC ++;;
			allC ++;

			if (sent.get(clc.get(0)).remainingAttr.containsKey("relation")
					&& !sent.get(clc.get(0)).remainingAttr.get("relation").equals(""))  {
				correctC++;
				
				if (clc.relation.equals(sent.get(clc.get(0)).remainingAttr.get("relation"))) correctCR++;

				if (clc.cl2_ind>=0 
						&& !sent.get(clc.cl2_ind).remainingAttr.get("old_clauseID").equals("") 
						&& sent.get(clc.cl2_ind).remainingAttr.get("old_clauseID")
						.equals(sent.get(clc.get(0)).remainingAttr.get("cl2"))) correctCRR++;

			} else {
				
			}
			
		}
		
	}
	
	
	public Sentence assignClauses(Sentence sent,
			ArrayList<ClauseBorder> clborders){
		
		int i = 0;
		int id = 0;
		
		ArrayList<Integer> idStack = new ArrayList<Integer>();
		
		for (int j = 0; j < sent.size(); j++){
			
			if (i<clborders.size() && j<clborders.get(i).position) {
								
				sent.get(j).clauseID = idStack.get(idStack.size()-1)+"";
				
			} else if (i<clborders.size() && j==clborders.get(i).position){
				
				
//				System.out.println("BORDER " + clborders.get(i).type);
					
				if (clborders.get(i).type.equals("CC")) {
					sent.get(j).clauseID = idStack.get(idStack.size()-1)+"";
					idStack.remove(idStack.size()-1);
					
					i++;
					
					while (i<clborders.size() && j==clborders.get(i).position
							&& clborders.get(i).type.contains("CC")) {

						idStack.remove(idStack.size()-1);
						i++;

					}


				} else {
					id++;
					idStack.add(id);
					
					sent.get(j).clauseID = idStack.get(idStack.size()-1)+"";
					
					i++;
					
					while (i<clborders.size() && j==clborders.get(i).position) {

						idStack.add(id);
						i++;

					}

				}
					
				
				
//				System.out.println("BORDER STACK " + idStack.size());
				
			} else {
				
				if (idStack.size() == 0){
					id++;
					idStack.add(id);

				}
				
				sent.get(j).clauseID = idStack.get(idStack.size()-1)+"";
			}
		}
		
		return sent;
	}


}
