package bg.bas.dcl.semantic.lexical_db;

import java.util.ArrayList;

import org.apache.commons.lang3.StringEscapeUtils;

public class Synset {
	
	public String ID = "";
	public String pos = "";
	public String definition = "";
	
	public String status = "";
	public String semclass = "";
	public String snote = "";
	public String stamp = "";
	
	public double positivity = 0.0;
	public double negativity = 0.0;
	
	public int bcs = 0;
	public int frequency = 0;
	public String domain = "";

	public ArrayList<Literal> literals = new ArrayList<Literal>();
	public ArrayList<Relation> relations = new ArrayList<Relation>();

	public ArrayList<String> usages = new ArrayList<String>();

	
	public String printLiterals(){
		
		String result = "";
		
		for (Literal l : literals) result += l.lemma + "; ";
		
		return result;
	}
	
	public String toXML(){
		
		if (pos.equals("r")) pos = "b";
		for (Relation r : relations){
			if (r.ID.endsWith("r")) r.ID.replace("-r", "-b");
		}
		
		String result = "<SYNSET>";
		
		result += "<ID>" + ID + "</ID>";
		result += "<POS>" + pos + "</POS>";
		result += "<SEM_CLASS>" + semclass + "</SEM_CLASS>";
		result += "<STAMP>" + stamp + "</STAMP>";
		result += "<BCS>" + bcs + "</BCS>";
		result += "<FREQUENCY>" + frequency + "</FREQUENCY>";
		result += "<DOMAIN>" + domain + "</DOMAIN>";
		result += "<POSITIVITY>" + positivity + "</POSITIVITY>";
		result += "<NEGATIVITY>" + negativity + "</NEGATIVITY>";

		result += "<SYNONYM>";
		
		for (Literal l : literals) {
			result += l.toXML();
		}
		
		result += "</SYNONYM>";

		
		for (Relation r : relations) {
			result += r.toXML();
		}
		
		

		for (String u : usages)	result += "<USAGE>" + StringEscapeUtils.escapeXml(u) + "</USAGE>";

		if (definition.length()>0) result += "<DEF>" + StringEscapeUtils.escapeXml(definition) + "</DEF>";
		if (status.length()>0) result += "<STATUS>" + StringEscapeUtils.escapeXml(status) + "</STATUS>";
		if (snote.length()>0) result += "<SNOTE>" + StringEscapeUtils.escapeXml(snote) + "</SNOTE>";


		result += "</SYNSET>";
		

		return result;

		
	}
	
	public boolean isEmpty(){
		
		if (status.contains("not")
				|| definition.contains("MT!")
				|| definition.contains("MTL!")
				|| definition.contains("Define me")) return true;
		else return false;

	}
		
	public String getRelation(String name) {
		for (Relation r : relations) {
			if (r.type.equalsIgnoreCase(name)) return r.sem;
		}
		return null;
	}
	
	
	public String getLiterals() {
		String res = "";
		for (Literal r : literals) {
			
			res += (r.lemma.equals("")?r.word:r.lemma) +":"+r.sense+ "; ";
		}
		return res;
	}

	public String getLiterals(boolean verifiedonly) {
		String res = "";
		for (Literal r : literals) {
			if (r.status.contains("not ")) continue;
			res += (r.lemma.equals("")?r.word:r.lemma) +":"+r.sense+ "; ";
		}
		return res;
	}
	
	public String getUsage() {
		String res = "";
		for ( String u : usages) {
			
			res += u+ "; ";
		}
		return res;
	}


	public String getHypernymID() {
		for (Relation r : relations) {
			if (r.type.equalsIgnoreCase("hypernym")) return r.ID;
		}
		return "";
	}
	
	
}
