package bg.bas.dcl.semantic.processing;

import java.io.File;
import java.io.FileNotFoundException;
import java.io.FileOutputStream;
import java.io.IOException;
import java.io.OutputStreamWriter;
import java.io.Writer;
import java.util.ArrayList;
import java.util.HashMap;
import java.util.Iterator;
import java.util.List;

import javax.xml.parsers.DocumentBuilder;
import javax.xml.parsers.DocumentBuilderFactory;

import org.w3c.dom.Document;
import org.w3c.dom.Element;
import org.w3c.dom.Node;
import org.w3c.dom.NodeList;

import edu.mit.jwi.item.ISynset;
import edu.mit.jwi.item.IWord;
import edu.mit.jwi.item.IWordID;
import edu.mit.jwi.item.POS;
import edu.mit.jwi.item.Pointer;
import bg.bas.dcl.entities.Sentence;
import bg.bas.dcl.entities.TextPosition;
import bg.bas.dcl.entities.VerbForm;
import bg.bas.dcl.entities.Word;
import bg.bas.dcl.general.FileHandler;
import bg.bas.dcl.general.InitialParametersReader;
import bg.bas.dcl.multilingual.Chunker.VerbFormRecogniser.VerbFormFormalism;
import bg.bas.dcl.multilingual.Chunker.VerbFormRecogniser.VerbFormRecogniser;
import bg.bas.dcl.multilingual.ClauseSplitting.RuleBasedClauseSplitter;
import bg.bas.dcl.semantic.anaphora.AnaphoraResolutor;
import bg.bas.dcl.semantic.lexical_db.MITJavaWordNetAccess;

public class SemanticProcessor {

	public HashMap<String,ArrayList<String>> parameters = new HashMap<String,ArrayList<String>>();

	Writer out ;
	VerbFormRecogniser vfr ;
	RuleBasedClauseSplitter rbcs ;
//	NounPhraseRecogniser nfr = new NounPhraseRecogniser();

	MITJavaWordNetAccess mitwn = new MITJavaWordNetAccess();
	
	PhraseModelSystem pmsystem ;
	
	public static void main(String[] args) throws FileNotFoundException{
		
		SemanticProcessor sp = new SemanticProcessor();
		
		// read from corpus BulSemCor
		
		// process:
		// - get sentence
		// - get verb forms
		// - get NPs - head 
		
		String inparfile = "/home/ivelina/WORK-DCL/"
				+ "SEMANTIC/sem-initparam.txt";
		sp.parameters = InitialParametersReader.readInitParameters(inparfile);
		
		sp.initialise();
		
		sp.out = new OutputStreamWriter(new FileOutputStream(
				sp.parameters.get("output-file").get(0))); 
		
		String dir = sp.parameters.get("input-folder").get(0);
		
//		sp.processDir(dir);
	
		sp.countDeriv();
	}
	
	

	
	public void countDeriv(){
		
		Iterator<ISynset> si = mitwn.dict.getSynsetIterator(POS.VERB);
		
		int ni = 0;
		
l1:		while (si.hasNext()){
			ISynset is = si.next();
			
			List<IWord> list = is.getWords();
			
			for (IWord w : list) {
				List<IWordID> llist = w.getRelatedWords(Pointer.DERIVATIONALLY_RELATED);
				
				for (IWordID wwid : llist){
					IWord ww = mitwn.dict.getWord(wwid);
					if (ww.getPOS().equals(POS.NOUN)){
						ni++;
						
						System.out.println(w.getLemma() + "\t" + ww.getLemma());
						continue l1;
					}
				}
//				
			}
			
		}
		System.out.println(ni);
	}
	
	public void initialise(){
		
		VerbFormFormalism vff = new VerbFormFormalism();
		vff.readModelsFromFile(
				parameters.get("bg-verbform-models-file")
					.get(0));	

		vfr = new VerbFormRecogniser(vff);
		
		pmsystem = new PhraseModelSystem(
				parameters.get("bg-npform-models-file")
				.get(0));
		
		/**
		System.out.println(vff.size());
		System.out.println((new File(parameters.get("bg-verbform-models-file")
				.get(0))).exists());
		System.out.println(parameters.get("bg-npform-models-file")
				.get(0));
		System.out.println((new File(parameters.get("bg-npform-models-file")
				.get(0))).exists());
		 */
		
		rbcs = new RuleBasedClauseSplitter (vff, 
				parameters.get("bg-conj-file")
				.get(0),
				parameters.get("bg-specialverbs-file")
				.get(0),
				parameters.get("punct-exceptions-file")
				.get(0));
	}
	
	public void processDir(String dir){
		try{
			FileHandler fh = new FileHandler();
			List<File> list = fh.getFileListing(new File(dir)); 
			
			for (File f : list) {
				
				if (!f.isFile()) continue;
				if (!f.getName().endsWith(".txt")) continue;
				
				processFile(f);
				
			}
//			out.flush();
//			out.close();

			for (String k : ctr.keySet()) {
				if (ctr.get(k)>3) {
					System.err.println(k+"\t"+ctr.get(k));
					
					allverbmodels.get(k).summerise();
					
					String fname = parameters.get("output-dir").get(0)
							+allverbmodels.get(k).synsetID+".html";
					out = new OutputStreamWriter(new FileOutputStream(
							fname,true)); 
					
					out.write("<br>\n*****<br><br>\n\n"+allverbmodels.get(k).getSummary());
					
					out.flush();
					out.close();
					
				} else {
					String fname = parameters.get("output-dir").get(0)
							+allverbmodels.get(k).synsetID+".html";
					File ff = new File(fname);
					if (ff.exists()) ff.delete();
					
				}
			}

			
		} catch(Exception e){
			e.printStackTrace();
		}	

	}

	public ArrayList<Sentence> getSentencesFromFile(File f){
		
		ArrayList<Sentence> sents = new ArrayList<Sentence>();
		HashMap<String,Integer> NElens = new HashMap<String,Integer>();

		try{


			DocumentBuilderFactory dbFactory = 
					DocumentBuilderFactory.newInstance();
			DocumentBuilder dBuilder = dbFactory.newDocumentBuilder();
			Document doc = dBuilder.parse(f);

			doc.getDocumentElement().normalize();
				 
			
			NodeList wordList = doc.getElementsByTagName("word");
			

			Sentence sent = new Sentence();
						
			for (int j = 0; j < wordList.getLength(); j++) {
				
				Node wordNode = wordList.item(j);
				
				if (wordNode.getNodeType() != Node.ELEMENT_NODE) continue;
				
				Element wordElement = (Element) wordNode;
				
				Word w = new Word(wordElement);
				
				/**
				w.wordform = wordElement.getAttribute("w");
				w.lemma = wordElement.getAttribute("l");
				w.wn = wordElement.getAttribute("wn");
				w.pos = wordElement.getAttribute("pos");
				
				if (wordElement.hasAttribute("u")) 
					w.remainingAttr.put("u", wordElement.getAttribute("u"));
				if (wordElement.hasAttribute("s")) 
					w.remainingAttr.put("s", wordElement.getAttribute("s"));
				if (wordElement.hasAttribute("t")) 
					w.remainingAttr.put("t", wordElement.getAttribute("t"));
				if (wordElement.hasAttribute("e")) 
					w.remainingAttr.put("e", wordElement.getAttribute("e"));

				 */
				
				if (w.lemma.equals("защо")) w.pos = "?"+w.pos;
				if (w.lemma.equals("как")) w.pos = "?"+w.pos;
				if (w.lemma.equals("къде")) w.pos = "?"+w.pos;
				if (w.lemma.equals("кога")) w.pos = "?"+w.pos;
				if (w.lemma.equals("когато")) w.pos = "?"+w.pos;
				if (w.lemma.equals("където")) w.pos = "?"+w.pos;
				if (w.lemma.equals("защото")) w.pos = "?"+w.pos;
				if (w.lemma.equals("както")) w.pos = "?"+w.pos;


				sent.add(w);
				
				if (w.remainingAttr.containsKey("e")
						&& w.remainingAttr.get("e").equalsIgnoreCase("true")) {
										
					sents.add(sent);
					sent = new Sentence();

				}

				
			}

					
		}catch(Exception e){
			e.printStackTrace();
		}
		
		return sents;
		
	}

	public void processFile(File f){
		
		try{
	
			System.out.println(f.getAbsolutePath());
	
			ArrayList<Sentence> sents = getSentencesFromFile(f);
	
//			System.out.println("SENTS: "+sents.size());
	
	
			sents = processSentences(sents);
			

			
					
		}catch(Exception e){
			e.printStackTrace();
		}
		
	}

	AnaphoraResolutor anres = new AnaphoraResolutor();
	
	HashMap<String,VerbArgumentModel> allverbmodels = new HashMap<String,VerbArgumentModel>();

	public ArrayList<Sentence> processSentences(ArrayList<Sentence> sents) throws IOException{
		
		ArrayList<VerbForm> allverbs = new ArrayList<VerbForm>();
		ArrayList<NounPhrase> allnouns = new ArrayList<NounPhrase>();
		
		int sentctr = 0;
		for (Sentence sent : sents){
			

			ArrayList<VerbForm> verbs = vfr.getAllForms(sent);
			ArrayList<NounPhrase> nouns = pmsystem.findForms(sent);
			
			for (int k=0; k< verbs.size(); k++){
				for (int l=0; l< verbs.get(k).size(); l++) {
					verbs.get(k).get(l).parpos = sentctr;
				}
			}
			for (int k=0; k< nouns.size(); k++){
				for (int l=0; l< nouns.get(k).size(); l++) {
					nouns.get(k).get(l).parpos = sentctr;
				}
			}
			
			sentctr++;
		
			allnouns.addAll(nouns);
			allverbs.addAll(verbs);
			
		}
		
		
		allnouns = anres.resolve(sents,allnouns);
		
		for (int i = 0; i<sents.size(); i++){
			
			Sentence sent = sents.get(i);
			
			ArrayList<VerbForm> verbs = new ArrayList<VerbForm>();
			ArrayList<NounPhrase> nouns = new ArrayList<NounPhrase>();
			
			for (VerbForm vf : allverbs) if (vf.get(0).parpos == i) verbs.add(vf);
			for (NounPhrase nf : allnouns) if (nf.get(0).parpos == i) nouns.add(nf);
			
			/**
			for (NounPhrase np : nouns) {
				if (np.referent != null) {
					System.out.println("REF: " + sent.get(np.get(0).sentpos).wordform
							+ "\t" + sents.get(np.referent.get(0).parpos).get(np.referent.get(0).sentpos).wordform);
				}
			}
			*/

//			writeColouredVerbFormsHTML(out, sent, verbs);
//			writeColouredNounPhrasesHTML(out, sent, nouns, verbs);
//			writeColouredNounPhrasesAnaphoraHTML(out, sents, sent, nouns, verbs);
			
			sent = rbcs.processSentence(sent);
			
//			for (Word w : sent) System.out.println(w.wordform+"/"+w.clauseID);

			catalogueFrames(parameters.get("output-dir").get(0), 
						sents, sent, nouns, verbs);
		
		}
		
		/**
		for (String k : ctr.keySet()) {
			if (ctr.get(k)>3) {
				System.err.println(k+"\t"+ctr.get(k));
				
				allverbmodels.get(k).summerise();
				
				String fname = parameters.get("output-dir").get(0)
						+allverbmodels.get(k).synsetID+".html";
				out = new OutputStreamWriter(new FileOutputStream(
						fname,true)); 
				
				out.write("\n*****\n\n"+allverbmodels.get(k).getSummary());
				
				out.flush();
				out.close();
				
			} else {
				String fname = parameters.get("output-dir").get(0)
						+allverbmodels.get(k).synsetID+".html";
				File f = new File(fname);
				if (f.exists()) f.delete();
				
			}
		}
		 */
		return sents;
	}
	
	HashMap<String,Integer> ctr = new HashMap<String,Integer>();
	
	public void catalogueFrames(String outdir, 
			ArrayList<Sentence> sents, 
			Sentence sent, 
			ArrayList<NounPhrase> nouns, 
			ArrayList<VerbForm> verbs){
		
		try {
			String result = "";
			
			
			for (int i = 0; i<verbs.size(); i++){
					
					VerbForm vf = verbs.get(i);
					String clID = sent.get(vf.get(0).sentpos).clauseID;
					
					Sentence clause = new Sentence();
					for (Word w : sent) {
						if (w.clauseID.equals(clID)) {
							clause.add(w);
							clause.plainText+= " "+w.wordform;
						}
					}
			
					String fname = outdir+vf.mainword.wn+".html";
					FileHandler.makeParent(fname, "");
					
					out = new OutputStreamWriter(new FileOutputStream(
							fname,true)); 
					
					if (!ctr.containsKey(vf.mainword.wn)){
						ctr.put(vf.mainword.wn, 0);
					}
					
					ctr.put(vf.mainword.wn, ctr.get(vf.mainword.wn)+1);
					
					VerbArgumentModel vam = 
							new VerbArgumentModel(
									vf.mainword.wn,sent,mitwn);
					
					if (allverbmodels.containsKey(vf.mainword.wn)){
						vam = allverbmodels.get(vf.mainword.wn);
					}
					
					VerbArgumentStructure struct = 
							new VerbArgumentStructure(vf);
					
//					for (Word w : sent) out.write(w.wordform+"/"+w.clauseID+" ");
//					out.write("\n");
//					for (VerbForm vv : verbs) out.write(vv.mainword.lemma+"\n");

					out.write(clause.plainText+"<br>\n");
					
					out.write("<b>");
					for (TextPosition tp : vf){
						out.write(sent.get(tp.sentpos).wordform+" ");
					}
					
					out.write("(" +vf.mainword.lemma+","+vf.mainword.wn+ ") ");
					
					if (vf.mainword!=null && !vf.mainword.wn.equals("")){
						try{
							out.write(mitwn.getLexicalName(vf.mainword.wn));
						} catch (Exception ex){}
					}
					
					out.write("</b>"+"<br>");
					out.write("\n"+"<br>");
					out.flush();
					
					for (NounPhrase np : nouns){
						
						if (!sent.get(np.get(0).sentpos).clauseID.equals(clID)) continue;
						
						VerbArgument va = new VerbArgument();
						
						if (np.get(0).sentpos < vf.get(0).sentpos) {
							out.write("PRE\t");
							va.PRE = true;
						} else {
							out.write("POST\t");
							va.PRE = false;
						}
						
						if (np.isPrepositional) {
							out.write(sent.get(np.get(0).sentpos).lemma+"\t");
							va.preposition = sent.get(np.get(0).sentpos).lemma;
						} else {
							out.write("NONE\t");
						}
						
						va.nounphrase = np;

						if (np.headPosition!=null
								&& np.headPosition.sentpos>-1) {
							out.write(sent.get(np.headPosition.sentpos).lemma);
							out.write("\t" + mitwn.getLexicalName(sent.get(np.headPosition.sentpos).wn));
							out.write("\n"+"<br>");

							va.head = sent.get(np.headPosition.sentpos);
							
						}
						
						struct.add(va);
						
					}
					
					vam.add(struct);
					allverbmodels.put(vf.mainword.wn,vam);
					
					out.write("\n"+"<br>");
					out.write("<hr>"+"<br>");
					out.write("\n"+"<br>");
					
					out.flush();
					out.close();
			}
			
			
			
		} catch (IOException e) {
			// TODO Auto-generated catch block
			e.printStackTrace();
		}
		
	}


	
	String[] colours = {
			"#0066FF",
			"#FF3300",
			"#999999",
			"#FFCC00",
			"#99CC00",
			"#CC00CC",
			"#009933",
			"#FF33CC",
			"#3300CC",
			"#CC0033",
			"#555500"
			};

	public void writeColouredNounPhrasesAnaphoraHTML(Writer out, 
			ArrayList<Sentence> sents, 
			Sentence sent, 
			ArrayList<NounPhrase> nouns, 
			ArrayList<VerbForm> verbs){
		
		String result = "";
				
			for (int i = 0; i<sent.size(); i++){
				
				
				
				boolean isHead = false;
				int v = -1;int ve = -1;
				for (int k = 0; k<nouns.size(); k++) {
					for (int l = 0; l<nouns.get(k).size(); l++){
						if (nouns.get(k).get(l).sentpos == i) {
							v = k;
							ve = nouns.get(k)
									.get(nouns.get(k).size()-1)
									.parpos;
							
						}
					}
				}

				int vb = -1;int vbe = -1;boolean isFullVerb = false;
				for (int k = 0; k<verbs.size(); k++) {
					for (int l = 0; l<verbs.get(k).size(); l++){
						if (verbs.get(k).get(l).sentpos == i) {
							vb = k;
							isFullVerb = !verbs.get(k).tense.get(0).startsWith("NVerb");
							vbe = verbs.get(k)
									.get(verbs.get(k).size()-1)
									.parpos;
						}
					}
				}

				
				Word t = sent.get(i);
			

				if (v>-1) result += "<a href=\"\" title=\""
								+ nouns.get(v).modelstring
								+"\"><font color=\"" 
								+ colours[v % colours.length] 
								+ "\">";

				if (v>-1 && nouns.get(v).headPosition.sentpos==i){
					result += "<b>";
				}
					
				if (vb>-1){
					result += "<a href=\"\" title=\""
							+ verbs.get(vb).tense.get(0)
							+"\"><font color=\"black\">";
					if (isFullVerb) result += "<b>";
				}

				result += t.wordform+"/"+t.pos+" ";

				if (v>-1 && nouns.get(v).headPosition.sentpos==i){
					result += "</b>";
				}

				if (v>-1) result += "</font></a> ";
				if (vb>-1){
					if (isFullVerb) result += "</b>";
					result += "</font></a>";
				}
				
				if (v>-1 && nouns.get(v).referent!=null 
						&& i==nouns.get(v).get(nouns.get(v).size()-1).sentpos) {
					result += " (";
					result = result.concat(getPhraseText(sents, nouns.get(v).referent));
					result += ")";
				}
				
				result += " ";

			}

		
		
		try {
			out.write(result+"<br><br>\n");
			out.flush();
			
		} catch (IOException e) {
			// TODO Auto-generated catch block
			e.printStackTrace();
		}
		
	}
	
	public String getPhraseText(ArrayList<Sentence> sents, NounPhrase phrase){
		
		String result = "";
		
		for (TextPosition tp : phrase) {
			Word w = sents.get(tp.parpos).get(tp.sentpos);
			if (w.pos.equals("R")) continue;
			result += w.wordform + " ";
		}
		
		return result.trim();
		
	}



	public void writeColouredNounPhrasesHTML(Writer out, 
			Sentence sent, 
			ArrayList<NounPhrase> nouns, 
			ArrayList<VerbForm> verbs){
		
		String result = "";
				
			for (int i = 0; i<sent.size(); i++){
				
				boolean isHead = false;
				int v = -1;int ve = -1;
				for (int k = 0; k<nouns.size(); k++) {
					for (int l = 0; l<nouns.get(k).size(); l++){
						if (nouns.get(k).get(l).sentpos == i) {
							v = k;
							ve = nouns.get(k)
									.get(nouns.get(k).size()-1)
									.parpos;
							
						}
					}
				}

				int vb = -1;int vbe = -1;boolean isFullVerb = false;
				for (int k = 0; k<verbs.size(); k++) {
					for (int l = 0; l<verbs.get(k).size(); l++){
						if (verbs.get(k).get(l).sentpos == i) {
							vb = k;
							isFullVerb = !verbs.get(k).tense.get(0).startsWith("NVerb");
							vbe = verbs.get(k)
									.get(verbs.get(k).size()-1)
									.parpos;
						}
					}
				}

				
				Word t = sent.get(i);
			

				if (v>-1) result += "<a href=\"\" title=\""
								+ nouns.get(v).modelstring
								+"\"><font color=\"" 
								+ colours[v % colours.length] 
								+ "\">";

				if (v>-1 && nouns.get(v).headPosition.sentpos==i){
					result += "<b>";
				}
					
				if (vb>-1){
					result += "<a href=\"\" title=\""
							+ verbs.get(vb).tense.get(0)
							+"\"><font color=\"black\">";
					if (isFullVerb) result += "<b>";
				}

				result += t.wordform+"/"+t.pos+" ";

				if (v>-1 && nouns.get(v).headPosition.sentpos==i){
					result += "</b>";
				}

				if (v>-1) result += "</font></a> ";
				if (vb>-1){
					if (isFullVerb) result += "</b>";
					result += "</font></a>";
				}
				
				if (v>-1 && i==ve) result += "<font color=\"" 
						+ colours[v % colours.length] 
						+ "\">"
						+ "</font> ";

				result += " ";

			}

		
		
		try {
			out.write(result+"<br><br>\n");
			out.flush();
			
		} catch (IOException e) {
			// TODO Auto-generated catch block
			e.printStackTrace();
		}
		
	}

	public void writeColouredNounPhrasesHTML(Writer out, 
			Sentence sent, ArrayList<NounPhrase> nouns){
		
		String result = "";
				
			for (int i = 0; i<sent.size(); i++){
				
				boolean isHead = false;
				int v = -1;int ve = -1;
				for (int k = 0; k<nouns.size(); k++) {
					for (int l = 0; l<nouns.get(k).size(); l++){
						if (nouns.get(k).get(l).sentpos == i) {
							v = k;
							ve = nouns.get(k)
									.get(nouns.get(k).size()-1)
									.parpos;
							
						}
					}
				}
				
				
				Word t = sent.get(i);
			

				if (v>-1) result += "<a href=\"\" title=\""
								+ nouns.get(v).modelstring
								+"\"><font color=\"" 
								+ colours[v % colours.length] 
								+ "\">";

				if (v>-1 && nouns.get(v).headPosition.sentpos==i){
					result += "<b>";
				}
					
				result += t.wordform+"/"+t.pos+" ";

				if (v>-1 && nouns.get(v).headPosition.sentpos==i){
					result += "</b>";
				}

				if (v>-1) result += "</font></a> ";
				
				if (v>-1 && i==ve) result += "<font color=\"" 
						+ colours[v % colours.length] 
						+ "\">"
						+ "</font> ";

				result += " ";

			}

		
		
		try {
			out.write(result+"<br><br>\n");
			out.flush();
			
		} catch (IOException e) {
			// TODO Auto-generated catch block
			e.printStackTrace();
		}
		
	}


}
